# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import json

import datasets


_DESCRIPTION = """\
HoVer is an open-domain, many-hop fact extraction and claim verification dataset built upon the Wikipedia corpus. The original 2-hop claims are adapted from question-answer pairs from HotpotQA. It is collected by a team of NLP researchers at UNC Chapel Hill and Verisk Analytics.
"""
_HOMEPAGE_URL = "https://hover-nlp.github.io/"
_CITATION = """\
@inproceedings{jiang2020hover,
  title={{HoVer}: A Dataset for Many-Hop Fact Extraction And Claim Verification},
  author={Yichen Jiang and Shikha Bordia and Zheng Zhong and Charles Dognin and Maneesh Singh and Mohit Bansal.},
  booktitle={Findings of the Conference on Empirical Methods in Natural Language Processing ({EMNLP})},
  year={2020}
}
"""

_TRAIN_URL = "https://raw.githubusercontent.com/hover-nlp/hover/main/data/hover/hover_train_release_v1.1.json"
_VALID_URL = "https://raw.githubusercontent.com/hover-nlp/hover/main/data/hover/hover_dev_release_v1.1.json"
_TEST_URL = "https://raw.githubusercontent.com/hover-nlp/hover/main/data/hover/hover_test_release_v1.1.json"


class Hover(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.1.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "uid": datasets.Value("string"),
                    "claim": datasets.Value("string"),
                    "supporting_facts": [
                        {
                            "key": datasets.Value("string"),
                            "value": datasets.Value("int32"),
                        }
                    ],
                    "label": datasets.ClassLabel(names=["NOT_SUPPORTED", "SUPPORTED"]),
                    "num_hops": datasets.Value("int32"),
                    "hpqa_id": datasets.Value("string"),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_URL)
        valid_path = dl_manager.download_and_extract(_VALID_URL)
        test_path = dl_manager.download_and_extract(_TEST_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": train_path, "datatype": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"datapath": valid_path, "datatype": "valid"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"datapath": test_path, "datatype": "test"},
            ),
        ]

    def _generate_examples(self, datapath, datatype):
        with open(datapath, encoding="utf-8") as f:
            data = json.load(f)

        for sentence_counter, d in enumerate(data):
            if datatype != "test":
                resp = {
                    "id": sentence_counter,
                    "uid": d["uid"],
                    "claim": d["claim"],
                    "supporting_facts": [{"key": x[0], "value": x[1]} for x in d["supporting_facts"]],
                    "label": d["label"],
                    "num_hops": d["num_hops"],
                    "hpqa_id": d["hpqa_id"],
                }
            else:
                resp = {
                    "id": sentence_counter,
                    "uid": d["uid"],
                    "claim": d["claim"],
                    "supporting_facts": [],
                    "label": -1,
                    "num_hops": -1,
                    "hpqa_id": "None",
                }
            yield sentence_counter, resp



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import os

import datasets


_CITATION = """\
@Misc{johnsonetal2014,
 author = {Johnson, Kyle P. and Patrick Burns and John Stewart and Todd Cook},
 title = {CLTK: The Classical Language Toolkit},
 url = {https://github.com/cltk/cltk},
 year = {2014--2020},
}
"""

_DESCRIPTION = """\
This dataset combines some of the classical Sanskrit texts.
"""

_HOMEPAGE = "https://github.com/parmarsuraj99/hf_datasets/tree/master/sanskrit_classic"

_LICENSE = ""

_URLs = {
    "combined": "https://github.com/parmarsuraj99/hf_datasets/raw/master/sanskrit_classic/combined.zip",
}


class SanskritClassic(datasets.GeneratorBasedBuilder):
    """TODO: Short description of my dataset."""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="combined", version=VERSION, description="This is combined version of classical texts"
        ),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "text": datasets.Value("string"),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "combined.txt"),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                yield id_, {
                    "text": row,
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Korean HateSpeech Dataset"""


import csv

import datasets


_CITATION = """\
@inproceedings{moon-etal-2020-beep,
    title = "{BEEP}! {K}orean Corpus of Online News Comments for Toxic Speech Detection",
    author = "Moon, Jihyung  and
      Cho, Won Ik  and
      Lee, Junbum",
    booktitle = "Proceedings of the Eighth International Workshop on Natural Language Processing for Social Media",
    month = jul,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.socialnlp-1.4",
    pages = "25--31",
    abstract = "Toxic comments in online platforms are an unavoidable social issue under the cloak of anonymity. Hate speech detection has been actively done for languages such as English, German, or Italian, where manually labeled corpus has been released. In this work, we first present 9.4K manually labeled entertainment news comments for identifying Korean toxic speech, collected from a widely used online news platform in Korea. The comments are annotated regarding social bias and hate speech since both aspects are correlated. The inter-annotator agreement Krippendorff{'}s alpha score is 0.492 and 0.496, respectively. We provide benchmarks using CharCNN, BiLSTM, and BERT, where BERT achieves the highest score on all tasks. The models generally display better performance on bias identification, since the hate speech detection is a more subjective issue. Additionally, when BERT is trained with bias label for hate speech detection, the prediction score increases, implying that bias and hate are intertwined. We make our dataset publicly available and open competitions with the corpus and benchmarks.",
}
"""

_DESCRIPTION = """\
Human-annotated Korean corpus collected from a popular domestic entertainment news aggregation platform
for toxic speech detection. Comments are annotated for gender bias, social bias and hate speech.
"""

_HOMEPAGE = "https://github.com/kocohub/korean-hate-speech"

_LICENSE = "Creative Commons"

_TRAIN_DOWNLOAD_URL = "https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/labeled/train.tsv"
_TEST_DOWNLOAD_URL = "https://raw.githubusercontent.com/kocohub/korean-hate-speech/master/labeled/dev.tsv"


class KorHate(datasets.GeneratorBasedBuilder):
    """Korean Corpus of Online News Comments for Toxic Speech Detection"""

    VERSION = datasets.Version("1.1.0")

    def _info(self):

        features = datasets.Features(
            {
                "comments": datasets.Value("string"),
                "contain_gender_bias": datasets.features.ClassLabel(names=["False", "True"]),
                "bias": datasets.features.ClassLabel(names=["none", "gender", "others"]),
                "hate": datasets.features.ClassLabel(names=["hate", "offensive", "none"]),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_DOWNLOAD_URL)
        test_path = dl_manager.download_and_extract(_TEST_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate Korean HateSpeech examples"""

        with open(filepath, encoding="utf-8") as tsv_file:
            tsv_reader = csv.DictReader(tsv_file, delimiter="\t", quoting=csv.QUOTE_NONE)
            for id_, row in enumerate(tsv_reader):
                yield id_, {
                    "comments": row["comments"],
                    "contain_gender_bias": row["contain_gender_bias"],
                    "bias": row["bias"],
                    "hate": row["hate"],
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""QED: A Dataset for Explanations in Question Answering"""


import json

import datasets


_CITATION = """\
@misc{lamm2020qed,
    title={QED: A Framework and Dataset for Explanations in Question Answering},
    author={Matthew Lamm and Jennimaria Palomaki and Chris Alberti and Daniel Andor and Eunsol Choi and Livio Baldini Soares and Michael Collins},
    year={2020},
    eprint={2009.06354},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
QED, is a linguistically informed, extensible framework for explanations in question answering. \
A QED explanation specifies the relationship between a question and answer according to formal semantic notions \
such as referential equality, sentencehood, and entailment. It is an expertannotated dataset of QED explanations \
built upon a subset of the Google Natural Questions dataset.
"""

_HOMEPAGE = "https://github.com/google-research-datasets/QED"

_BASE_URL = "https://raw.githubusercontent.com/google-research-datasets/QED/master/"
_URLS = {
    "train": _BASE_URL + "qed-train.jsonlines",
    "dev": _BASE_URL + "qed-dev.jsonlines",
}


class Qed(datasets.GeneratorBasedBuilder):
    """QED: A Dataset for Explanations in Question Answering"""

    VERSION = datasets.Version("1.0.0")
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="qed", version=datasets.Version("1.0.0")),
    ]

    def _info(self):
        span_features = {
            "start": datasets.Value("int32"),
            "end": datasets.Value("int32"),
            "string": datasets.Value("string"),
        }
        reference_features = {
            "start": datasets.Value("int32"),
            "end": datasets.Value("int32"),
            "bridge": datasets.Value("string"),
            "string": datasets.Value("string"),
        }
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "example_id": datasets.Value("int64"),
                    "title_text": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "question": datasets.Value("string"),
                    "paragraph_text": datasets.Value("string"),
                    "sentence_starts": datasets.Sequence(datasets.Value("int32")),
                    "original_nq_answers": [span_features],
                    "annotation": {
                        "referential_equalities": [
                            {
                                "question_reference": span_features,
                                "sentence_reference": reference_features,
                            }
                        ],
                        "answer": [
                            {
                                "sentence_reference": reference_features,
                                "paragraph_reference": span_features,
                            }
                        ],
                        "explanation_type": datasets.Value("string"),
                        "selected_sentence": span_features,
                    },
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        downloaded_paths = dl_manager.download(_URLS)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": downloaded_paths["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": downloaded_paths["dev"]},
            ),
        ]

    def _generate_examples(self, filepath):
        with open(filepath, encoding="utf-8") as f:
            examples = f.readlines()
            for example in examples:
                example = json.loads(example.strip())
                example["question"] = example.pop("question_text")

                # some examples have missing annotation, assign empty values to such examples
                if "answer" not in example["annotation"]:
                    example["annotation"]["answer"] = []
                if "selected_sentence" not in example["annotation"]:
                    example["annotation"]["selected_sentence"] = {
                        "start": -1,
                        "end": -1,
                        "string": "",
                    }
                if "referential_equalities" not in example["annotation"]:
                    example["annotation"]["referential_equalities"] = []
                else:
                    for referential_equalities in example["annotation"]["referential_equalities"]:
                        bridge = referential_equalities["sentence_reference"]["bridge"]
                        referential_equalities["sentence_reference"]["bridge"] = (
                            bridge if bridge is not False else None
                        )

                # remove the nested list
                example["original_nq_answers"] = example["original_nq_answers"][0]

                yield example["example_id"], example



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Social Bias Frames"""


import csv

import datasets


_CITATION = """\
@inproceedings{sap2020socialbiasframes,
   title={Social Bias Frames: Reasoning about Social and Power Implications of Language},
   author={Sap, Maarten and Gabriel, Saadia and Qin, Lianhui and Jurafsky, Dan and Smith, Noah A and Choi, Yejin},
   year={2020},
   booktitle={ACL},
}
"""

_DESCRIPTION = """\
Social Bias Frames is a new way of representing the biases and offensiveness that are implied in language.
For example, these frames are meant to distill the implication that "women (candidates) are less qualified"
behind the statement "we shouldn’t lower our standards to hire more women."
"""

_DATA_URL = "https://homes.cs.washington.edu/~msap/social-bias-frames/SBIC.v2.tgz"


class SocialBiasFrames(datasets.GeneratorBasedBuilder):
    """TSocial Bias Frame"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "whoTarget": datasets.Value("string"),
                    "intentYN": datasets.Value("string"),
                    "sexYN": datasets.Value("string"),
                    "sexReason": datasets.Value("string"),
                    "offensiveYN": datasets.Value("string"),
                    "annotatorGender": datasets.Value("string"),
                    "annotatorMinority": datasets.Value("string"),
                    "sexPhrase": datasets.Value("string"),
                    "speakerMinorityYN": datasets.Value("string"),
                    "WorkerId": datasets.Value("string"),
                    "HITId": datasets.Value("string"),
                    "annotatorPolitics": datasets.Value("string"),
                    "annotatorRace": datasets.Value("string"),
                    "annotatorAge": datasets.Value("string"),
                    "post": datasets.Value("string"),
                    "targetMinority": datasets.Value("string"),
                    "targetCategory": datasets.Value("string"),
                    "targetStereotype": datasets.Value("string"),
                    "dataSource": datasets.Value("string"),
                }
            ),
            # No default supervised_keys (as we have to pass both premise
            # and hypothesis as input).
            supervised_keys=None,
            homepage="https://homes.cs.washington.edu/~msap/social-bias-frames/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        archive = dl_manager.download(_DATA_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": "SBIC.v2.tst.csv", "files": dl_manager.iter_archive(archive)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": "SBIC.v2.dev.csv", "files": dl_manager.iter_archive(archive)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": "SBIC.v2.trn.csv", "files": dl_manager.iter_archive(archive)},
            ),
        ]

    def _generate_examples(self, filepath, files):
        """This function returns the examples in the raw (text) form."""
        for path, f in files:
            if path == filepath:
                lines = (line.decode("utf-8") for line in f)
                reader = csv.DictReader(lines)
                for idx, row in enumerate(reader):
                    yield idx, row
                break



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""The GermEval 2014 NER Shared Task dataset."""


import csv

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{benikova-etal-2014-nosta,
    title = {NoSta-D Named Entity Annotation for German: Guidelines and Dataset},
    author = {Benikova, Darina  and
      Biemann, Chris  and
      Reznicek, Marc},
    booktitle = {Proceedings of the Ninth International Conference on Language Resources and Evaluation ({LREC}'14)},
    month = {may},
    year = {2014},
    address = {Reykjavik, Iceland},
    publisher = {European Language Resources Association (ELRA)},
    url = {http://www.lrec-conf.org/proceedings/lrec2014/pdf/276_Paper.pdf},
    pages = {2524--2531},
}
"""

_DESCRIPTION = """\
The GermEval 2014 NER Shared Task builds on a new dataset with German Named Entity annotation with the following properties:\
    - The data was sampled from German Wikipedia and News Corpora as a collection of citations.\
    - The dataset covers over 31,000 sentences corresponding to over 590,000 tokens.\
    - The NER annotation uses the NoSta-D guidelines, which extend the Tübingen Treebank guidelines,\
      using four main NER categories with sub-structure, and annotating embeddings among NEs\
      such as [ORG FC Kickers [LOC Darmstadt]].
"""

_URLS = {
    "train": "https://drive.google.com/uc?export=download&id=1Jjhbal535VVz2ap4v4r_rN1UEHTdLK5P",
    "dev": "https://drive.google.com/uc?export=download&id=1ZfRcQThdtAR5PPRjIDtrVP7BtXSCUBbm",
    "test": "https://drive.google.com/uc?export=download&id=1u9mb7kNJHWQCWyweMDRMuTFoOHOfeBTH",
}


class GermEval14Config(datasets.BuilderConfig):
    """BuilderConfig for GermEval 2014."""

    def __init__(self, **kwargs):
        """BuilderConfig for GermEval 2014.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(GermEval14Config, self).__init__(**kwargs)


class GermEval14(datasets.GeneratorBasedBuilder):
    """GermEval 2014 NER Shared Task dataset."""

    BUILDER_CONFIGS = [
        GermEval14Config(
            name="germeval_14", version=datasets.Version("2.0.0"), description="GermEval 2014 NER Shared Task dataset"
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "source": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-LOC",
                                "I-LOC",
                                "B-LOCderiv",
                                "I-LOCderiv",
                                "B-LOCpart",
                                "I-LOCpart",
                                "B-ORG",
                                "I-ORG",
                                "B-ORGderiv",
                                "I-ORGderiv",
                                "B-ORGpart",
                                "I-ORGpart",
                                "B-OTH",
                                "I-OTH",
                                "B-OTHderiv",
                                "I-OTHderiv",
                                "B-OTHpart",
                                "I-OTHpart",
                                "B-PER",
                                "I-PER",
                                "B-PERderiv",
                                "I-PERderiv",
                                "B-PERpart",
                                "I-PERpart",
                            ]
                        )
                    ),
                    "nested_ner_tags": datasets.Sequence(
                        datasets.features.ClassLabel(
                            names=[
                                "O",
                                "B-LOC",
                                "I-LOC",
                                "B-LOCderiv",
                                "I-LOCderiv",
                                "B-LOCpart",
                                "I-LOCpart",
                                "B-ORG",
                                "I-ORG",
                                "B-ORGderiv",
                                "I-ORGderiv",
                                "B-ORGpart",
                                "I-ORGpart",
                                "B-OTH",
                                "I-OTH",
                                "B-OTHderiv",
                                "I-OTHderiv",
                                "B-OTHpart",
                                "I-OTHpart",
                                "B-PER",
                                "I-PER",
                                "B-PERderiv",
                                "I-PERderiv",
                                "B-PERpart",
                                "I-PERpart",
                            ]
                        )
                    ),
                }
            ),
            supervised_keys=None,
            homepage="https://sites.google.com/site/germeval2014ner/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_files = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            data = csv.reader(f, delimiter="\t", quoting=csv.QUOTE_NONE)
            current_source = ""
            current_tokens = []
            current_ner_tags = []
            current_nested_ner_tags = []
            sentence_counter = 0
            for row in data:
                if row:
                    if row[0] == "#":
                        current_source = " ".join(row[1:])
                        continue
                    id_, token, label, nested_label = row[:4]
                    current_tokens.append(token)
                    current_ner_tags.append(label)
                    current_nested_ner_tags.append(nested_label)
                else:
                    # New sentence
                    if not current_tokens:
                        # Consecutive empty lines will cause empty sentences
                        continue
                    assert len(current_tokens) == len(current_ner_tags), "💔 between len of tokens & labels"
                    assert len(current_ner_tags) == len(
                        current_nested_ner_tags
                    ), "💔 between len of labels & nested labels"
                    assert current_source, "💥 Source for new sentence was not set"
                    sentence = (
                        sentence_counter,
                        {
                            "id": str(sentence_counter),
                            "tokens": current_tokens,
                            "ner_tags": current_ner_tags,
                            "nested_ner_tags": current_nested_ner_tags,
                            "source": current_source,
                        },
                    )
                    sentence_counter += 1
                    current_tokens = []
                    current_ner_tags = []
                    current_nested_ner_tags = []
                    current_source = ""
                    yield sentence
            # Don't forget last sentence in dataset 🧐
            yield sentence_counter, {
                "id": str(sentence_counter),
                "tokens": current_tokens,
                "ner_tags": current_ner_tags,
                "nested_ner_tags": current_nested_ner_tags,
                "source": current_source,
            }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Asian Language Treebank (ALT) Project"""


import os

import datasets


_CITATION = """\
@inproceedings{riza2016introduction,
  title={Introduction of the asian language treebank},
  author={Riza, Hammam and Purwoadi, Michael and Uliniansyah, Teduh and Ti, Aw Ai and Aljunied, Sharifah Mahani and Mai, Luong Chi and Thang, Vu Tat and Thai, Nguyen Phuong and Chea, Vichet and Sam, Sethserey and others},
  booktitle={2016 Conference of The Oriental Chapter of International Committee for Coordination and Standardization of Speech Databases and Assessment Techniques (O-COCOSDA)},
  pages={1--6},
  year={2016},
  organization={IEEE}
}
"""

_HOMEPAGE = "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/"

_DESCRIPTION = """\
The ALT project aims to advance the state-of-the-art Asian natural language processing (NLP) techniques through the open collaboration for developing and using ALT. It was first conducted by NICT and UCSY as described in Ye Kyaw Thu, Win Pa Pa, Masao Utiyama, Andrew Finch and Eiichiro Sumita (2016). Then, it was developed under ASEAN IVO as described in this Web page. The process of building ALT began with sampling about 20,000 sentences from English Wikinews, and then these sentences were translated into the other languages. ALT now has 13 languages: Bengali, English, Filipino, Hindi, Bahasa Indonesia, Japanese, Khmer, Lao, Malay, Myanmar (Burmese), Thai, Vietnamese, Chinese (Simplified Chinese).
"""

_URLs = {
    "alt": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/ALT-Parallel-Corpus-20191206.zip",
    "alt-en": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/English-ALT-20170107.zip",
    "alt-jp": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/Japanese-ALT-20170330.zip",
    "alt-my": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/my-alt-190530.zip",
    "alt-my-transliteration": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/my-en-transliteration.zip",
    "alt-my-west-transliteration": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/western-myanmar-transliteration.zip",
    "alt-km": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/km-nova-181101.zip",
}

_SPLIT = {
    "train": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/URL-train.txt",
    "dev": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/URL-dev.txt",
    "test": "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/URL-test.txt",
}

_WIKI_URL = "https://www2.nict.go.jp/astrec-att/member/mutiyama/ALT/ALT-Parallel-Corpus-20191206/URL.txt"


class AltParallelConfig(datasets.BuilderConfig):
    """BuilderConfig for ALT."""

    def __init__(self, languages, **kwargs):
        """BuilderConfig for ALT.

        Args:
            for the `datasets.features.text.TextEncoder` used for the features feature.

            languages: languages that will be used for translation. it should be one of the
            **kwargs: keyword arguments forwarded to super.
        """

        name = "alt-parallel"

        description = "ALT Parallel Corpus"
        super(AltParallelConfig, self).__init__(
            name=name,
            description=description,
            version=datasets.Version("1.0.0", ""),
            **kwargs,
        )

        available_langs = set(
            ["bg", "en", "en_tok", "fil", "hi", "id", "ja", "khm", "lo", "ms", "my", "th", "vi", "zh"]
        )
        for language in languages:
            assert language in available_langs

        self.languages = languages


class Alt(datasets.GeneratorBasedBuilder):
    """Asian Language Treebank (ALT) Project"""

    BUILDER_CONFIGS = [
        AltParallelConfig(
            languages=["bg", "en", "en_tok", "fil", "hi", "id", "ja", "khm", "lo", "ms", "my", "th", "vi", "zh"]
        ),
        datasets.BuilderConfig(name="alt-en", version=datasets.Version("1.0.0"), description="English ALT"),
        datasets.BuilderConfig(name="alt-jp", version=datasets.Version("1.0.0"), description="Japanese ALT"),
        datasets.BuilderConfig(name="alt-my", version=datasets.Version("1.0.0"), description="Myanmar ALT"),
        datasets.BuilderConfig(name="alt-km", version=datasets.Version("1.0.0"), description="Khmer ALT"),
        datasets.BuilderConfig(
            name="alt-my-transliteration",
            version=datasets.Version("1.0.0"),
            description="Myanmar-English Transliteration Dataset",
        ),
        datasets.BuilderConfig(
            name="alt-my-west-transliteration",
            version=datasets.Version("1.0.0"),
            description="Latin-Myanmar Transliteration Dataset",
        ),
    ]

    DEFAULT_CONFIG_NAME = "alt-parallel"

    def _info(self):
        if self.config.name.startswith("alt-parallel"):
            features = datasets.Features(
                {
                    "SNT.URLID": datasets.Value("string"),
                    "SNT.URLID.SNTID": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "translation": datasets.features.Translation(languages=self.config.languages),
                }
            )
        elif self.config.name == "alt-en":
            features = datasets.Features(
                {
                    "SNT.URLID": datasets.Value("string"),
                    "SNT.URLID.SNTID": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "status": datasets.Value("string"),
                    "value": datasets.Value("string"),
                }
            )
        elif self.config.name == "alt-jp":
            features = datasets.Features(
                {
                    "SNT.URLID": datasets.Value("string"),
                    "SNT.URLID.SNTID": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "status": datasets.Value("string"),
                    "value": datasets.Value("string"),
                    "word_alignment": datasets.Value("string"),
                    "jp_tokenized": datasets.Value("string"),
                    "en_tokenized": datasets.Value("string"),
                }
            )
        elif self.config.name == "alt-my":
            features = datasets.Features(
                {
                    "SNT.URLID": datasets.Value("string"),
                    "SNT.URLID.SNTID": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "value": datasets.Value("string"),
                }
            )
        elif self.config.name == "alt-my-transliteration":
            features = datasets.Features(
                {
                    "en": datasets.Value("string"),
                    "my": datasets.Sequence(datasets.Value("string")),
                }
            )
        elif self.config.name == "alt-my-west-transliteration":
            features = datasets.Features(
                {
                    "en": datasets.Value("string"),
                    "my": datasets.Sequence(datasets.Value("string")),
                }
            )
        elif self.config.name == "alt-km":
            features = datasets.Features(
                {
                    "SNT.URLID": datasets.Value("string"),
                    "SNT.URLID.SNTID": datasets.Value("string"),
                    "url": datasets.Value("string"),
                    "km_pos_tag": datasets.Value("string"),
                    "km_tokenized": datasets.Value("string"),
                }
            )
        else:
            raise

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        if self.config.name.startswith("alt-parallel"):
            data_path = dl_manager.download_and_extract(_URLs["alt"])
        else:
            data_path = dl_manager.download_and_extract(_URLs[self.config.name])

        if self.config.name == "alt-my-transliteration" or self.config.name == "alt-my-west-transliteration":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"basepath": data_path, "split": None},
                )
            ]
        else:
            data_split = {}
            for k in _SPLIT:
                data_split[k] = dl_manager.download_and_extract(_SPLIT[k])

            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={"basepath": data_path, "split": data_split["train"]},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    gen_kwargs={"basepath": data_path, "split": data_split["dev"]},
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={"basepath": data_path, "split": data_split["test"]},
                ),
            ]

    def _generate_examples(self, basepath, split=None):
        allow_urls = {}
        if split is not None:
            with open(split, encoding="utf-8") as fin:
                for line in fin:
                    sp = line.strip().split("\t")
                    urlid = sp[0].replace("URL.", "")
                    allow_urls[urlid] = {"SNT.URLID": urlid, "url": sp[1]}

        data = {}
        if self.config.name.startswith("alt-parallel"):
            files = self.config.languages

            data = {}
            for lang in files:
                file_path = os.path.join(basepath, "ALT-Parallel-Corpus-20191206", f"data_{lang}.txt")
                fin = open(file_path, encoding="utf-8")
                for line in fin:
                    line = line.strip()
                    sp = line.split("\t")

                    _, urlid, sntid = sp[0].split(".")
                    if urlid not in allow_urls:
                        continue

                    if sntid not in data:
                        data[sntid] = {}
                        data[sntid]["SNT.URLID"] = urlid
                        data[sntid]["SNT.URLID.SNTID"] = sntid
                        data[sntid]["url"] = allow_urls[urlid]["url"]
                        data[sntid]["translation"] = {}

                    # Note that Japanese and Myanmar texts have empty sentence fields in this release.
                    if len(sp) >= 2:
                        data[sntid]["translation"][lang] = sp[1]

                fin.close()

        elif self.config.name == "alt-en":
            data = {}
            for fname in ["English-ALT-Draft.txt", "English-ALT-Reviewed.txt"]:
                file_path = os.path.join(basepath, "English-ALT-20170107", fname)
                fin = open(file_path, encoding="utf-8")
                for line in fin:
                    line = line.strip()
                    sp = line.split("\t")

                    _, urlid, sntid = sp[0].split(".")
                    if urlid not in allow_urls:
                        continue

                    d = {
                        "SNT.URLID": urlid,
                        "SNT.URLID.SNTID": sntid,
                        "url": allow_urls[urlid]["url"],
                        "status": None,
                        "value": None,
                    }

                    d["value"] = sp[1]
                    if fname == "English-ALT-Draft.txt":
                        d["status"] = "draft"
                    else:
                        d["status"] = "reviewed"

                    data[sntid] = d
                fin.close()
        elif self.config.name == "alt-jp":
            data = {}
            for fname in ["Japanese-ALT-Draft.txt", "Japanese-ALT-Reviewed.txt"]:
                file_path = os.path.join(basepath, "Japanese-ALT-20170330", fname)
                fin = open(file_path, encoding="utf-8")
                for line in fin:
                    line = line.strip()
                    sp = line.split("\t")
                    _, urlid, sntid = sp[0].split(".")
                    if urlid not in allow_urls:
                        continue

                    d = {
                        "SNT.URLID": urlid,
                        "SNT.URLID.SNTID": sntid,
                        "url": allow_urls[urlid]["url"],
                        "value": None,
                        "status": None,
                        "word_alignment": None,
                        "en_tokenized": None,
                        "jp_tokenized": None,
                    }

                    d["value"] = sp[1]
                    if fname == "Japanese-ALT-Draft.txt":
                        d["status"] = "draft"
                    else:
                        d["status"] = "reviewed"
                    data[sntid] = d
                fin.close()

            keys = {
                "word_alignment": "word-alignment/data_ja.en-ja",
                "en_tokenized": "word-alignment/data_ja.en-tok",
                "jp_tokenized": "word-alignment/data_ja.ja-tok",
            }
            for k in keys:
                file_path = os.path.join(basepath, "Japanese-ALT-20170330", keys[k])
                fin = open(file_path, encoding="utf-8")
                for line in fin:
                    line = line.strip()
                    sp = line.split("\t")

                    # Note that Japanese and Myanmar texts have empty sentence fields in this release.
                    if len(sp) < 2:
                        continue

                    _, urlid, sntid = sp[0].split(".")
                    if urlid not in allow_urls:
                        continue

                    if sntid in data:

                        data[sntid][k] = sp[1]
                fin.close()

        elif self.config.name == "alt-my":
            data = {}
            for fname in ["data"]:
                file_path = os.path.join(basepath, "my-alt-190530", fname)
                fin = open(file_path, encoding="utf-8")
                for line in fin:
                    line = line.strip()
                    sp = line.split("\t")
                    _, urlid, sntid = sp[0].split(".")
                    if urlid not in allow_urls:
                        continue

                    data[sntid] = {
                        "SNT.URLID": urlid,
                        "SNT.URLID.SNTID": sntid,
                        "url": allow_urls[urlid]["url"],
                        "value": sp[1],
                    }
                fin.close()

        elif self.config.name == "alt-km":
            data = {}
            for fname in ["data_km.km-tag.nova", "data_km.km-tok.nova"]:
                file_path = os.path.join(basepath, "km-nova-181101", fname)
                fin = open(file_path, encoding="utf-8")
                for line in fin:
                    line = line.strip()
                    sp = line.split("\t")
                    _, urlid, sntid = sp[0].split(".")
                    if urlid not in allow_urls:
                        continue

                    k = "km_pos_tag" if fname == "data_km.km-tag.nova" else "km_tokenized"
                    if sntid in data:
                        data[sntid][k] = sp[1]
                    else:
                        data[sntid] = {
                            "SNT.URLID": urlid,
                            "SNT.URLID.SNTID": sntid,
                            "url": allow_urls[urlid]["url"],
                            "km_pos_tag": None,
                            "km_tokenized": None,
                        }
                        data[sntid][k] = sp[1]
                fin.close()

        elif self.config.name == "alt-my-transliteration":
            file_path = os.path.join(basepath, "my-en-transliteration", "data.txt")
            # Need to set errors='ignore' because of the unknown error
            # UnicodeDecodeError: 'utf-8' codec can't decode byte 0xff in position 0: invalid start byte
            # It might due to some issues related to Myanmar alphabets
            fin = open(file_path, encoding="utf-8", errors="ignore")
            _id = 0
            for line in fin:
                line = line.strip()

                # I don't know why there are \x00 between |||. They don't show in the editor.
                line = line.replace("\x00", "")
                sp = line.split("|||")

                # When I read data, it seems to have empty sentence betweem the actual sentence. Don't know why?
                if len(sp) < 2:
                    continue

                data[_id] = {"en": sp[0].strip(), "my": [sp[1].strip()]}
                _id += 1
            fin.close()
        elif self.config.name == "alt-my-west-transliteration":
            file_path = os.path.join(basepath, "western-myanmar-transliteration", "321.txt")
            # Need to set errors='ignore' because of the unknown error
            # UnicodeDecodeError: 'utf-8' codec can't decode byte 0xff in position 0: invalid start byte
            # It might due to some issues related to Myanmar alphabets
            fin = open(file_path, encoding="utf-8", errors="ignore")
            _id = 0
            for line in fin:
                line = line.strip()
                line = line.replace("\x00", "")
                sp = line.split("|||")

                data[_id] = {"en": sp[0].strip(), "my": [k.strip() for k in sp[1].split("|")]}
                _id += 1
            fin.close()

        _id = 1
        for k in data:
            yield _id, data[k]
            _id += 1



import copy
from dataclasses import dataclass
from itertools import cycle, islice, repeat
from typing import Any, Callable, Dict, Iterable, Iterator, List, Optional, Union

import numpy as np
import pyarrow as pa

from .arrow_dataset import DatasetInfoMixin
from .features import Features, FeatureType
from .formatting import PythonFormatter
from .info import DatasetInfo
from .splits import NamedSplit
from .table import table_cast


def _infer_features_from_batch(batch: Dict[str, list], try_features: Optional[Features] = None) -> Features:
    pa_table = pa.Table.from_pydict(batch)
    if try_features is not None:
        try:
            pa_table = table_cast(pa_table, pa.schema(try_features.type))
        except (TypeError, pa.ArrowInvalid, pa.ArrowNotImplementedError):
            pass
    return Features.from_arrow_schema(pa_table.schema)


def _examples_to_batch(examples: List[Dict[str, Any]]) -> Dict[str, list]:
    cols = sorted(examples[0].keys())
    arrays = []
    for col in cols:
        arrays.append([example[col] for example in examples])
    return dict(zip(cols, arrays))


def _batch_to_examples(batch: Dict[str, list]) -> List[Dict[str, Any]]:
    """Convert a batch (dict of examples) to examples list"""
    n_examples = len(batch[next(iter(batch))])
    for i in range(n_examples):
        yield {col: array[i] for col, array in batch.items()}


class _BaseExamplesIterable:
    """Base class for the examples iterable used by an IterableDataset"""

    def __iter__(self):
        """An examples iterable should yield tuples (example_key, example) of type (int/str, dict)"""
        raise NotImplementedError()

    def shuffle_data_sources(self, seed: Optional[int]) -> "_BaseExamplesIterable":
        """
        Either shuffle the shards/sources of the dataset, or propagate the shuffling to the underlying iterable.
        If the order of the shards must stay fixed (when using .skip or .take for example), then this method returns self.
        """
        raise NotImplementedError()

    @property
    def n_shards(self) -> int:
        raise NotImplementedError()


def _shuffle_kwargs(rng: np.random.Generator, kwargs: dict) -> dict:
    shuffled_kwargs = {}
    for key, value in sorted(kwargs.items()):
        if isinstance(value, list):
            value = list(value)
            rng.shuffle(value)
            shuffled_kwargs[key] = value
        else:
            shuffled_kwargs[key] = value
    return shuffled_kwargs


class ExamplesIterable(_BaseExamplesIterable):
    def __init__(self, generate_examples_fn: Callable, kwargs: dict):
        self.generate_examples_fn = generate_examples_fn
        self.kwargs = kwargs

    def __iter__(self):
        for key, example in self.generate_examples_fn(**self.kwargs):
            yield key, example

    def shuffle_data_sources(self, seed: Optional[int]) -> "ExamplesIterable":
        return ShardShuffledExamplesIterable(self.generate_examples_fn, self.kwargs, seed)

    @property
    def n_shards(self) -> int:
        max_length = max([len(value) for value in self.kwargs.values() if isinstance(value, list)], default=0)
        return max(1, max_length)


class ShardShuffledExamplesIterable(ExamplesIterable):
    def __init__(self, generate_examples_fn: Callable, kwargs: dict, seed: Optional[int]):
        super().__init__(generate_examples_fn, kwargs)
        self.seed = seed

    def __iter__(self):
        """Shuffle the kwargs order to shuffle shards"""
        rng = np.random.default_rng(self.seed)
        kwargs_with_shuffled_shards = _shuffle_kwargs(rng, self.kwargs)
        for key, example in self.generate_examples_fn(**kwargs_with_shuffled_shards):
            yield key, example


class CyclingMultiSourcesExamplesIterable(_BaseExamplesIterable):
    def __init__(self, ex_iterables: List[_BaseExamplesIterable]):
        self.ex_iterables = ex_iterables

    def __iter__(self):
        iterators = [iter(ex_iterable) for ex_iterable in self.ex_iterables]
        # this is an infinite iterator to keep track of which iterator we want to pick examples from
        indices_iterator = cycle(range(len(iterators)))
        for i in indices_iterator:
            try:  # let's pick one example from the iterator at index i
                yield next(iterators[i])
            except StopIteration:  # if we ran out of examples on this iterator, break the main for loop
                break

    def shuffle_data_sources(self, seed: Optional[int]) -> "CyclingMultiSourcesExamplesIterable":
        """Shuffle each underlying examples iterable."""
        ex_iterables = [ex_iterable.shuffle_data_sources(seed) for ex_iterable in self.ex_iterables]
        return CyclingMultiSourcesExamplesIterable(ex_iterables)

    @property
    def n_shards(self) -> int:
        return sum(ex_iterable.n_shards for ex_iterable in self.ex_iterables)


class RandomlyCyclingMultiSourcesExamplesIterable(CyclingMultiSourcesExamplesIterable):
    def __init__(self, ex_iterables, seed: Optional[int] = None, probabilities: Optional[List[float]] = None):
        super().__init__(ex_iterables)
        self.seed = seed
        self.probabilities = probabilities

    @staticmethod
    def _iter_random_indices(
        rng: np.random.Generator,
        num_sources: int,
        random_batch_size=1000,
        p: Optional[List[float]] = None,
    ) -> Iterator[int]:
        """Get an infinite iterator that randomly samples the index of the source to pick examples from."""
        if p is None:
            while True:
                yield from (int(i) for i in rng.integers(0, num_sources, size=random_batch_size))
        else:
            while True:
                yield from (int(i) for i in rng.choice(num_sources, size=random_batch_size, p=p))

    def __iter__(self):
        rng = np.random.default_rng(self.seed)
        iterators = [iter(ex_iterable) for ex_iterable in self.ex_iterables]
        # this is an infinite iterator that randomly samples the index of the source to pick examples from
        indices_iterator = self._iter_random_indices(rng, len(iterators), p=self.probabilities)
        for i in indices_iterator:
            try:  # let's pick one example from the iterator at index i
                yield next(iterators[i])
            except StopIteration:  # if we ran out of examples on this iterator, break the main for loop
                break

    def shuffle_data_sources(self, seed: Optional[int]) -> "RandomlyCyclingMultiSourcesExamplesIterable":
        """Shuffle the data sources of each wrapped examples iterable."""
        ex_iterables = [ex_iterable.shuffle_data_sources(seed) for ex_iterable in self.ex_iterables]
        return RandomlyCyclingMultiSourcesExamplesIterable(ex_iterables, seed=seed, probabilities=self.probabilities)


class MappedExamplesIterable(_BaseExamplesIterable):
    def __init__(
        self, ex_iterable: _BaseExamplesIterable, function: Callable, batched: bool = False, batch_size: int = 1000
    ):
        self.ex_iterable = ex_iterable
        self.function = function
        self.batched = batched
        self.batch_size = batch_size

    def __iter__(self):
        iterator = iter(self.ex_iterable)
        for key, example in iterator:
            if self.batched:
                # If batched, first build the batch
                key_examples_list = [(key, example)] + [
                    (key, example) for key, example in islice(iterator, self.batch_size - 1)
                ]
                keys, examples = zip(*key_examples_list)
                batch = _examples_to_batch(examples)
                # then apply the transform
                transformed_batch = self.function(batch)
                # the new key is the concatenation of the examples keys from the batch
                new_key = "_".join(str(key) for key in keys)
                # yield one example at a time from the transformed batch
                yield from zip(repeat(new_key), _batch_to_examples(transformed_batch))
            else:
                # If not batched, apply the transform and yield the example directly
                yield key, self.function(example)

    def shuffle_data_sources(self, seed: Optional[int]) -> "MappedExamplesIterable":
        """Shuffle the wrapped examples iterable."""
        return MappedExamplesIterable(
            self.ex_iterable.shuffle_data_sources(seed),
            function=self.function,
            batched=self.batched,
            batch_size=self.batch_size,
        )

    @property
    def n_shards(self) -> int:
        return self.ex_iterable.n_shards


class BufferShuffledExamplesIterable(_BaseExamplesIterable):
    def __init__(self, ex_iterable: _BaseExamplesIterable, buffer_size: int, seed: Optional[int]):
        self.ex_iterable = ex_iterable
        self.buffer_size = buffer_size
        self.seed = seed

    @staticmethod
    def _iter_random_indices(rng: np.random.Generator, buffer_size: int, random_batch_size=1000) -> Iterator[int]:
        while True:
            yield from (int(i) for i in rng.integers(0, buffer_size, size=random_batch_size))

    def __iter__(self):
        buffer_size = self.buffer_size
        rng = np.random.default_rng(self.seed)
        indices_iterator = self._iter_random_indices(rng, buffer_size)
        # this is the shuffle buffer that we keep in memory
        mem_buffer = []
        for x in self.ex_iterable:
            if len(mem_buffer) == buffer_size:  # if the buffer is full, pick and example from it
                i = next(indices_iterator)
                yield mem_buffer[i]
                mem_buffer[i] = x  # replace the picked example by a new one
            else:  # otherwise, keep filling the buffer
                mem_buffer.append(x)
        # when we run out of examples, we shuffle the remaining examples in the buffer and yield them
        rng.shuffle(mem_buffer)
        yield from mem_buffer

    def shuffle_data_sources(self, seed: Optional[int]) -> "BufferShuffledExamplesIterable":
        """Shuffle the wrapped examples iterable as well as the shuffling buffer."""
        return BufferShuffledExamplesIterable(
            self.ex_iterable.shuffle_data_sources(seed), buffer_size=self.buffer_size, seed=seed
        )

    @property
    def n_shards(self) -> int:
        return self.ex_iterable.n_shards


class SkipExamplesIterable(_BaseExamplesIterable):
    def __init__(self, ex_iterable: _BaseExamplesIterable, n: int):
        self.ex_iterable = ex_iterable
        self.n = n

    def __iter__(self):
        ex_iterator = iter(self.ex_iterable)
        for _ in islice(ex_iterator, self.n):
            pass
        yield from ex_iterator

    def shuffle_data_sources(self, seed: Optional[int]) -> "SkipExamplesIterable":
        """Doesn't shuffle the wrapped examples iterable since it would skip exampels from other shards instead."""
        return self

    @property
    def n_shards(self) -> int:
        return self.ex_iterable.n_shards


class TakeExamplesIterable(_BaseExamplesIterable):
    def __init__(self, ex_iterable: _BaseExamplesIterable, n: int):
        self.ex_iterable = ex_iterable
        self.n = n

    def __iter__(self):
        yield from islice(self.ex_iterable, self.n)

    def shuffle_data_sources(self, seed: Optional[int]) -> "TakeExamplesIterable":
        """Doesn't shuffle the wrapped examples iterable since it would take examples from other shards instead."""
        return self

    @property
    def n_shards(self) -> int:
        return self.ex_iterable.n_shards


class TypedExamplesIterable(_BaseExamplesIterable):
    def __init__(self, ex_iterable: _BaseExamplesIterable, features: Features):
        self.ex_iterable = ex_iterable
        self.features = features

    def __iter__(self):
        for key, example in self.ex_iterable:
            # we encode the example for ClassLabel feature types for example
            encoded_example = self.features.encode_example(example)
            # Decode example for Audio feature, e.g.
            decoded_example = self.features.decode_example(encoded_example)
            yield key, decoded_example

    def shuffle_data_sources(self, seed: Optional[int]) -> "TypedExamplesIterable":
        """Shuffle the wrapped examples iterable."""
        return TypedExamplesIterable(
            self.ex_iterable.shuffle_data_sources(seed),
            features=self.features,
        )

    @property
    def n_shards(self) -> int:
        return self.ex_iterable.n_shards


def _generate_examples_from_tables_wrapper(generate_tables_fn):
    def wrapper(**kwargs):
        python_formatter = PythonFormatter()
        for key, table in generate_tables_fn(**kwargs):
            batch = python_formatter.format_batch(table)
            for i, example in enumerate(_batch_to_examples(batch)):
                yield f"{key}_{i}", example

    return wrapper


@dataclass
class ShufflingConfig:
    seed: Optional[int] = None


class IterableDataset(DatasetInfoMixin):
    """A Dataset backed by an iterable."""

    def __init__(
        self,
        ex_iterable: _BaseExamplesIterable,
        info: Optional[DatasetInfo] = None,
        split: Optional[NamedSplit] = None,
        format_type: Optional[str] = None,
        shuffling: Optional[ShufflingConfig] = None,
    ):
        info = info.copy() if info is not None else DatasetInfo()
        DatasetInfoMixin.__init__(self, info=info, split=split)

        self._ex_iterable = ex_iterable
        self._format_type = format_type
        self._shuffling = shuffling
        self._epoch = 0

    def _head(self, n=5):
        return _examples_to_batch([x for key, x in islice(self._iter(), n)])

    @property
    def _effective_seed(self):
        if self._shuffling:
            return self._shuffling.seed + self._epoch if self._shuffling.seed is not None else None
        else:
            return None

    @property
    def n_shards(self) -> int:
        return self._ex_iterable.n_shards

    def _iter(self):
        if self._shuffling:
            ex_iterable = self._ex_iterable.shuffle_data_sources(self._effective_seed)
        else:
            ex_iterable = self._ex_iterable
        yield from ex_iterable

    def __iter__(self):
        for key, example in self._iter():
            if self.features:
                # we encode the example for ClassLabel feature types for example
                encoded_example = self.features.encode_example(example)
                # Decode example for Audio feature, e.g.
                decoded_example = self.features.decode_example(encoded_example)
                yield decoded_example
            else:
                yield example

    def with_format(
        self,
        type: Optional[str] = None,
    ) -> "IterableDataset":
        """
        Return a dataset with the specified format.
        This method only supports the "torch" format for now.

        Args:

            type (:obj:`str`, optional, default None): if set to "torch", the returned dataset
                will be a subclass of torch.utils.data.IterableDataset to be used in a DataLoader
        """
        # TODO(QL): add examples formatting to get tensors when using the "torch" format
        # TODO(QL): add format_kwargs
        # TODO(QL): add format_columns and return_all_columns
        # TODO(QL): add pandas, numpy and tf formats
        return iterable_dataset(
            ex_iterable=self._ex_iterable,
            info=copy.deepcopy(self._info),
            split=self._split,
            format_type=type,
            shuffling=copy.deepcopy(self._shuffling),
        )

    def map(self, function: Callable, batched: bool = False, batch_size: int = 1000):
        """
        Return a dataset with the specified map function. The function is applied on-the-fly on the examples when iterating over the dataset.

        You can specify whether the function should be batched or not with the ``batched`` parameter:

        - If batched is False, then the function takes 1 example in and should return 1 example.
          An example is a dictionary, e.g. {"text": "Hello there !"}
        - If batched is True and batch_size is 1, then the function takes a batch of 1 example as input and can return a batch with 1 or more examples.
          A batch is a dictionary, e.g. a batch of 1 example is {"text": ["Hello there !"]}
        - If batched is True and batch_size is ``n`` > 1, then the function takes a batch of ``n`` examples as input and can return a batch with ``n`` examples, or with an arbitrary number of examples.
          Note that the last batch may have less than ``n`` examples.
          A batch is a dictionary, e.g. a batch of ``n`` examples is {"text": ["Hello there !"] * n}

        Args:
            function (:obj:`Callable`, optional, default None): if not None, this function is applied
                on-the-fly on the examples when you iterate on the dataset.
            batched (:obj:`bool`, default `False`): Provide batch of examples to `function`.
            batch_size (:obj:`int`, optional, default ``1000``): Number of examples per batch provided to `function` if `batched=True`.

        """
        info = copy.deepcopy(self._info)
        info.features = None
        ex_iterable = MappedExamplesIterable(
            TypedExamplesIterable(self._ex_iterable, self._info.features)
            if self._info.features is not None
            else self._ex_iterable,
            function=function,
            batched=batched,
            batch_size=batch_size,
        )
        return iterable_dataset(
            ex_iterable=ex_iterable,
            info=info,
            split=self._split,
            format_type=self._format_type,
            shuffling=copy.deepcopy(self._shuffling),
        )

    def shuffle(self, buffer_size, seed=None) -> "IterableDataset":
        """
        Randomly shuffles the elements of this dataset.

        This dataset fills a buffer with buffer_size elements, then randomly samples elements from this buffer,
        replacing the selected elements with new elements. For perfect shuffling, a buffer size greater than or
        equal to the full size of the dataset is required.

        For instance, if your dataset contains 10,000 elements but ``buffer_size`` is set to 1,000, then shuffle will
        initially select a random element from only the first 1,000 elements in the buffer. Once an element is
        selected, its space in the buffer is replaced by the next (i.e. 1,001-st) element,
        maintaining the 1,000 element buffer.

        If the dataset is made of several shards, it also does shuffle the order of the shards.
        However if the order has been fixed by using :func:`datasets.IterableDataset.skip` or :func:`datasets.IterableDataset.take`
        then the order of the shards is kept unchanged.

        Args:
            buffer_size (:obj:`int`): size of the buffer.
            seed (:obj:`int`, optional, default None): random seed that will be used to create the distribution.
        """
        shuffling = ShufflingConfig(seed=seed)
        return iterable_dataset(
            ex_iterable=BufferShuffledExamplesIterable(self._ex_iterable, buffer_size, seed=seed).shuffle_data_sources(
                seed=seed
            ),
            info=copy.deepcopy(self._info),
            split=self._split,
            format_type=self._format_type,
            shuffling=shuffling,
        )

    def set_epoch(self, epoch: int):
        self._epoch = epoch

    def skip(self, n) -> "IterableDataset":
        """
        Create a new IterableDataset that skips the first ``n`` elements.

        Args:
            n (:obj:`int`): number of elements to skip.
        """
        ex_iterable = SkipExamplesIterable(self._ex_iterable, n)
        return iterable_dataset(
            ex_iterable=ex_iterable,
            info=copy.deepcopy(self._info),
            split=self._split,
            format_type=self._format_type,
            shuffling=copy.deepcopy(self._shuffling),
        )

    def take(self, n) -> "IterableDataset":
        """
        Create a new IterableDataset with only the first ``n`` elements.

        Args:
            n (:obj:`int`): number of elements to take.
        """
        ex_iterable = TakeExamplesIterable(self._ex_iterable, n)
        return iterable_dataset(
            ex_iterable=ex_iterable,
            info=copy.deepcopy(self._info),
            split=self._split,
            format_type=self._format_type,
            shuffling=copy.deepcopy(self._shuffling),
        )

    def remove_columns(self, column_names: Union[str, List[str]]) -> "IterableDataset":
        """
        Remove one or several column(s) in the dataset and the features associated to them.
        The removal is done on-the-fly on the examples when iterating over the dataset.


        Args:
            column_names (:obj:`Union[str, List[str]]`): Name of the column(s) to remove.

        Returns:
            :class:`IterableDataset`: A copy of the dataset object without the columns to remove.
        """
        if isinstance(column_names, str):
            column_names = [column_names]

        def remove_fn(example):
            return {k: v for k, v in example.items() if k not in column_names}

        return self.map(remove_fn)

    def cast_column(self, column: str, feature: FeatureType) -> "IterableDataset":
        """Cast column to feature for decoding.

        Args:
            column (:obj:`str`): Column name.
            feature (:class:`Feature`): Target feature.

        Returns:
            :class:`IterableDataset`
        """
        info = copy.deepcopy(self._info)
        info.features[column] = feature
        return iterable_dataset(
            ex_iterable=self._ex_iterable,
            info=info,
            split=self._split,
            format_type=self._format_type,
            shuffling=copy.deepcopy(self._shuffling),
        )


def iterable_dataset(
    ex_iterable: Iterable,
    info: Optional[DatasetInfo] = None,
    split: Optional[NamedSplit] = None,
    format_type: Optional[str] = None,
    shuffling: Optional[ShufflingConfig] = None,
):
    if format_type is not None and format_type == "torch":
        import torch

        class TorchIterableDataset(IterableDataset, torch.utils.data.IterableDataset):
            pass

        cls = TorchIterableDataset
    else:
        cls = IterableDataset
    return cls(
        ex_iterable=ex_iterable,
        info=info,
        split=split,
        format_type=format_type,
        shuffling=shuffling,
    )



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""GoEmotions dataset"""


import csv
import os

import datasets


_DESCRIPTION = """\
The GoEmotions dataset contains 58k carefully curated Reddit comments labeled for 27 emotion categories or Neutral.
The emotion categories are admiration, amusement, anger, annoyance, approval, caring, confusion, curiosity, desire,
disappointment, disapproval, disgust, embarrassment, excitement, fear, gratitude, grief, joy, love, nervousness,
optimism, pride, realization, relief, remorse, sadness, surprise.
"""

_CITATION = """\
@inproceedings{demszky2020goemotions,
 author = {Demszky, Dorottya and Movshovitz-Attias, Dana and Ko, Jeongwoo and Cowen, Alan and Nemade, Gaurav and Ravi, Sujith},
 booktitle = {58th Annual Meeting of the Association for Computational Linguistics (ACL)},
 title = {{GoEmotions: A Dataset of Fine-Grained Emotions}},
 year = {2020}
}
"""

_CLASS_NAMES = [
    "admiration",
    "amusement",
    "anger",
    "annoyance",
    "approval",
    "caring",
    "confusion",
    "curiosity",
    "desire",
    "disappointment",
    "disapproval",
    "disgust",
    "embarrassment",
    "excitement",
    "fear",
    "gratitude",
    "grief",
    "joy",
    "love",
    "nervousness",
    "optimism",
    "pride",
    "realization",
    "relief",
    "remorse",
    "sadness",
    "surprise",
    "neutral",
]

_BASE_DOWNLOAD_URL = "https://github.com/google-research/google-research/raw/master/goemotions/data/"
_RAW_DOWNLOAD_URLS = [
    "https://storage.googleapis.com/gresearch/goemotions/data/full_dataset/goemotions_1.csv",
    "https://storage.googleapis.com/gresearch/goemotions/data/full_dataset/goemotions_2.csv",
    "https://storage.googleapis.com/gresearch/goemotions/data/full_dataset/goemotions_3.csv",
]
_HOMEPAGE = "https://github.com/google-research/google-research/tree/master/goemotions"


class GoEmotionsConfig(datasets.BuilderConfig):
    @property
    def features(self):
        if self.name == "simplified":
            return {
                "text": datasets.Value("string"),
                "labels": datasets.Sequence(datasets.ClassLabel(names=_CLASS_NAMES)),
                "id": datasets.Value("string"),
            }
        elif self.name == "raw":
            d = {
                "text": datasets.Value("string"),
                "id": datasets.Value("string"),
                "author": datasets.Value("string"),
                "subreddit": datasets.Value("string"),
                "link_id": datasets.Value("string"),
                "parent_id": datasets.Value("string"),
                "created_utc": datasets.Value("float"),
                "rater_id": datasets.Value("int32"),
                "example_very_unclear": datasets.Value("bool"),
            }
            d.update({label: datasets.Value("int32") for label in _CLASS_NAMES})
            return d


class GoEmotions(datasets.GeneratorBasedBuilder):
    """GoEmotions dataset"""

    BUILDER_CONFIGS = [
        GoEmotionsConfig(
            name="raw",
        ),
        GoEmotionsConfig(
            name="simplified",
        ),
    ]
    BUILDER_CONFIG_CLASS = GoEmotionsConfig
    DEFAULT_CONFIG_NAME = "simplified"

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(self.config.features),
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        if self.config.name == "raw":
            paths = dl_manager.download_and_extract(_RAW_DOWNLOAD_URLS)
            return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepaths": paths, "raw": True})]
        if self.config.name == "simplified":
            train_path = dl_manager.download_and_extract(os.path.join(_BASE_DOWNLOAD_URL, "train.tsv"))
            dev_path = dl_manager.download_and_extract(os.path.join(_BASE_DOWNLOAD_URL, "dev.tsv"))
            test_path = dl_manager.download_and_extract(os.path.join(_BASE_DOWNLOAD_URL, "test.tsv"))
            return [
                datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepaths": [train_path]}),
                datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepaths": [dev_path]}),
                datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepaths": [test_path]}),
            ]

    def _generate_examples(self, filepaths, raw=False):
        """Generate AG News examples."""
        for file_idx, filepath in enumerate(filepaths):
            with open(filepath, "r", encoding="utf-8") as f:
                if raw:
                    reader = csv.DictReader(f)
                else:
                    reader = csv.DictReader(f, delimiter="\t", fieldnames=list(self.config.features.keys()))

                for row_idx, row in enumerate(reader):
                    if raw:
                        row["example_very_unclear"] = row["example_very_unclear"] == "TRUE"
                    else:
                        row["labels"] = [int(ind) for ind in row["labels"].split(",")]

                    yield f"{file_idx}_{row_idx}", row



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Roman Urdu data corpus with 20,000 polarity labeled records"""


import csv
import os

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@InProceedings{Sharf:2018,
title = "Performing Natural Language Processing on Roman Urdu Datasets",
authors = "Zareen Sharf and Saif Ur Rahman",
booktitle = "International Journal of Computer Science and Network Security",
volume = "18",
number = "1",
pages = "141-148",
year = "2018"
}

@misc{Dua:2019,
author = "Dua, Dheeru and Graff, Casey",
year = "2017",
title = "{UCI} Machine Learning Repository",
url = "http://archive.ics.uci.edu/ml",
institution = "University of California, Irvine, School of Information and Computer Sciences"
}
"""

_DESCRIPTION = """\
This is an extensive compilation of Roman Urdu Dataset (Urdu written in Latin/Roman script) tagged for sentiment analysis.
"""

_HOMEPAGE = "https://archive.ics.uci.edu/ml/datasets/Roman+Urdu+Data+Set"

_URL = "https://archive.ics.uci.edu/ml/machine-learning-databases/00458/Roman%20Urdu%20DataSet.csv"


class RomanUrdu(datasets.GeneratorBasedBuilder):
    """Roman Urdu sentences gathered from reviews of various e-commerce websites, comments on public Facebook pages, and twitter accounts, with positive, neutral, and negative polarity labels per each row."""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "sentiment": datasets.features.ClassLabel(names=["Positive", "Negative", "Neutral"]),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="sentence", label_column="sentiment")],
        )

    def _split_generators(self, dl_manager):
        data_dir = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": os.path.join(data_dir),
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        with open(filepath, encoding="utf-8") as f:
            reader = csv.reader(f, delimiter=",")
            for id_, row in enumerate(reader):
                yield id_, {
                    "sentence": row[0],
                    # 'Neative' typo in original dataset
                    "sentiment": "Negative" if row[1] == "Neative" else row[1],
                }



import csv

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@inproceedings{saravia-etal-2018-carer,
    title = "{CARER}: Contextualized Affect Representations for Emotion Recognition",
    author = "Saravia, Elvis  and
      Liu, Hsien-Chi Toby  and
      Huang, Yen-Hao  and
      Wu, Junlin  and
      Chen, Yi-Shin",
    booktitle = "Proceedings of the 2018 Conference on Empirical Methods in Natural Language Processing",
    month = oct # "-" # nov,
    year = "2018",
    address = "Brussels, Belgium",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/D18-1404",
    doi = "10.18653/v1/D18-1404",
    pages = "3687--3697",
    abstract = "Emotions are expressed in nuanced ways, which varies by collective or individual experiences, knowledge, and beliefs. Therefore, to understand emotion, as conveyed through text, a robust mechanism capable of capturing and modeling different linguistic nuances and phenomena is needed. We propose a semi-supervised, graph-based algorithm to produce rich structural descriptors which serve as the building blocks for constructing contextualized affect representations from text. The pattern-based representations are further enriched with word embeddings and evaluated through several emotion recognition tasks. Our experimental results demonstrate that the proposed method outperforms state-of-the-art techniques on emotion recognition tasks.",
}
"""

_DESCRIPTION = """\
Emotion is a dataset of English Twitter messages with six basic emotions: anger, fear, joy, love, sadness, and surprise. For more detailed information please refer to the paper.
"""
_URL = "https://github.com/dair-ai/emotion_dataset"
# use dl=1 to force browser to download data instead of displaying it
_TRAIN_DOWNLOAD_URL = "https://www.dropbox.com/s/1pzkadrvffbqw6o/train.txt?dl=1"
_VALIDATION_DOWNLOAD_URL = "https://www.dropbox.com/s/2mzialpsgf9k5l3/val.txt?dl=1"
_TEST_DOWNLOAD_URL = "https://www.dropbox.com/s/ikkqxfdbdec3fuj/test.txt?dl=1"


class Emotion(datasets.GeneratorBasedBuilder):
    def _info(self):
        class_names = ["sadness", "joy", "love", "anger", "fear", "surprise"]
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"text": datasets.Value("string"), "label": datasets.ClassLabel(names=class_names)}
            ),
            supervised_keys=("text", "label"),
            homepage=_URL,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        train_path = dl_manager.download_and_extract(_TRAIN_DOWNLOAD_URL)
        valid_path = dl_manager.download_and_extract(_VALIDATION_DOWNLOAD_URL)
        test_path = dl_manager.download_and_extract(_TEST_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": valid_path}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(csv_file, delimiter=";")
            for id_, row in enumerate(csv_reader):
                text, label = row
                yield id_, {"text": text, "label": label}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""COVID-19 Japanese Tweets Dataset."""


import bz2
import csv

import datasets


_CITATION = """\
No paper about this dataset is published yet. \
Please cite this dataset as "鈴木 優: COVID-19 日本語 Twitter データセット （http://www.db.info.gifu-u.ac.jp/covid-19-twitter-dataset/）"
"""

_DESCRIPTION = """\
53,640 Japanese tweets with annotation if a tweet is related to COVID-19 or not. The annotation is by majority decision by 5 - 10 crowd workers. \
Target tweets include "COVID" or "コロナ". The period of the tweets is from around January 2020 to around June 2020. \
The original tweets are not contained. Please use Twitter API to get them, for example.
"""

_HOMEPAGE = "http://www.db.info.gifu-u.ac.jp/covid-19-twitter-dataset/"

_LICENSE = "CC-BY-ND 4.0"

# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URLs = {
    "url": "http://www.db.info.gifu-u.ac.jp/data/covid19.csv.bz2",
}


class CovidTweetsJapanese(datasets.GeneratorBasedBuilder):
    """COVID-19 Japanese Tweets Dataset."""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        features = datasets.Features(
            {
                "tweet_id": datasets.Value("string"),
                "assessment_option_id": datasets.ClassLabel(names=["63", "64", "65", "66", "67", "68"]),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        my_urls = _URLs["url"]
        # data_url = dl_manager.download_and_extract(my_urls)
        data_url = dl_manager.download(my_urls)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": data_url, "split": "train"},
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""

        with bz2.open(filepath, "rt") as f:
            data = csv.reader(f)
            _ = next(data)
            for id_, row in enumerate(data):
                yield id_, {
                    "tweet_id": row[0],
                    "assessment_option_id": row[1],
                }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Multilingual Disaster Response Messages dataset."""


import csv

import datasets


_DESCRIPTION = """\
This dataset contains 30,000 messages drawn from events including an earthquake in Haiti in 2010, an earthquake in Chile in 2010, floods in Pakistan in 2010, super-storm Sandy in the U.S.A. in 2012, and news articles spanning a large number of years and 100s of different disasters.
The data has been encoded with 36 different categories related to disaster response and has been stripped of messages with sensitive information in their entirety.
Upon release, this is the featured dataset of a new Udacity course on Data Science and the AI4ALL summer school and is especially utile for text analytics and natural language processing (NLP) tasks and models.
The input data in this job contains thousands of untranslated disaster-related messages and their English translations.
"""

_CITATION = """\
@inproceedings{title={Multilingual Disaster Response Messages}
}
"""

_TRAIN_DOWNLOAD_URL = "https://s3.amazonaws.com/datasets.huggingface.co/disaster_response_messages_training.csv"

_TEST_DOWNLOAD_URL = "https://s3.amazonaws.com/datasets.huggingface.co/disaster_response_messages_test.csv"

_VALID_DOWNLOAD_URL = "https://s3.amazonaws.com/datasets.huggingface.co/disaster_response_messages_validation.csv"


class DisasterResponseMessages(datasets.GeneratorBasedBuilder):
    """Multilingual Disaster Response Messages dataset."""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "split": datasets.Value("string"),
                    "message": datasets.Value("string"),
                    "original": datasets.Value("string"),
                    "genre": datasets.Value("string"),
                    "related": datasets.ClassLabel(names=["false", "true", "maybe"]),
                    "PII": datasets.Value("int8"),
                    "request": datasets.ClassLabel(names=["false", "true"]),
                    "offer": datasets.Value("int8"),
                    "aid_related": datasets.ClassLabel(names=["false", "true"]),
                    "medical_help": datasets.ClassLabel(names=["false", "true"]),
                    "medical_products": datasets.ClassLabel(names=["false", "true"]),
                    "search_and_rescue": datasets.ClassLabel(names=["false", "true"]),
                    "security": datasets.ClassLabel(names=["false", "true"]),
                    "military": datasets.ClassLabel(names=["false", "true"]),
                    "child_alone": datasets.Value("int8"),
                    "water": datasets.ClassLabel(names=["false", "true"]),
                    "food": datasets.ClassLabel(names=["false", "true"]),
                    "shelter": datasets.ClassLabel(names=["false", "true"]),
                    "clothing": datasets.ClassLabel(names=["false", "true"]),
                    "money": datasets.ClassLabel(names=["false", "true"]),
                    "missing_people": datasets.ClassLabel(names=["false", "true"]),
                    "refugees": datasets.ClassLabel(names=["false", "true"]),
                    "death": datasets.ClassLabel(names=["false", "true"]),
                    "other_aid": datasets.ClassLabel(names=["false", "true"]),
                    "infrastructure_related": datasets.ClassLabel(names=["false", "true"]),
                    "transport": datasets.ClassLabel(names=["false", "true"]),
                    "buildings": datasets.ClassLabel(names=["false", "true"]),
                    "electricity": datasets.ClassLabel(names=["false", "true"]),
                    "tools": datasets.ClassLabel(names=["false", "true"]),
                    "hospitals": datasets.ClassLabel(names=["false", "true"]),
                    "shops": datasets.ClassLabel(names=["false", "true"]),
                    "aid_centers": datasets.ClassLabel(names=["false", "true"]),
                    "other_infrastructure": datasets.ClassLabel(names=["false", "true"]),
                    "weather_related": datasets.ClassLabel(names=["false", "true"]),
                    "floods": datasets.ClassLabel(names=["false", "true"]),
                    "storm": datasets.ClassLabel(names=["false", "true"]),
                    "fire": datasets.ClassLabel(names=["false", "true"]),
                    "earthquake": datasets.ClassLabel(names=["false", "true"]),
                    "cold": datasets.ClassLabel(names=["false", "true"]),
                    "other_weather": datasets.ClassLabel(names=["false", "true"]),
                    "direct_report": datasets.ClassLabel(names=["false", "true"]),
                }
            ),
            homepage="https://appen.com/datasets/combined-disaster-response-data/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_path, test_path, valid_path = dl_manager.download_and_extract(
            [_TRAIN_DOWNLOAD_URL, _TEST_DOWNLOAD_URL, _VALID_DOWNLOAD_URL]
        )
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_path}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": valid_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate Distaster Response Messages examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )
            next(csv_reader, None)
            for id_, row in enumerate(csv_reader):
                row = row[1:]
                (
                    split,
                    message,
                    original,
                    genre,
                    related,
                    PII,
                    request,
                    offer,
                    aid_related,
                    medical_help,
                    medical_products,
                    search_and_rescue,
                    security,
                    military,
                    child_alone,
                    water,
                    food,
                    shelter,
                    clothing,
                    money,
                    missing_people,
                    refugees,
                    death,
                    other_aid,
                    infrastructure_related,
                    transport,
                    buildings,
                    electricity,
                    tools,
                    hospitals,
                    shops,
                    aid_centers,
                    other_infrastructure,
                    weather_related,
                    floods,
                    storm,
                    fire,
                    earthquake,
                    cold,
                    other_weather,
                    direct_report,
                ) = row

                yield id_, {
                    "split": (split),
                    "message": (message),
                    "original": (original),
                    "genre": (genre),
                    "related": int(related),
                    "PII": int(PII),
                    "request": int(request),
                    "offer": int(offer),
                    "aid_related": int(aid_related),
                    "medical_help": int(medical_help),
                    "medical_products": int(medical_products),
                    "search_and_rescue": int(search_and_rescue),
                    "security": int(security),
                    "military": int(military),
                    "child_alone": int(child_alone),
                    "water": int(water),
                    "food": int(food),
                    "shelter": int(shelter),
                    "clothing": int(clothing),
                    "money": int(money),
                    "missing_people": int(missing_people),
                    "refugees": int(refugees),
                    "death": int(death),
                    "other_aid": int(other_aid),
                    "infrastructure_related": int(infrastructure_related),
                    "transport": int(transport),
                    "buildings": int(buildings),
                    "electricity": int(electricity),
                    "tools": int(tools),
                    "hospitals": int(hospitals),
                    "shops": int(shops),
                    "aid_centers": int(aid_centers),
                    "other_infrastructure": int(other_infrastructure),
                    "weather_related": int(weather_related),
                    "floods": int(floods),
                    "storm": int(storm),
                    "fire": int(fire),
                    "earthquake": int(earthquake),
                    "cold": int(cold),
                    "other_weather": int(other_weather),
                    "direct_report": int(direct_report),
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Multi-Dimensional Gender Bias classification"""


import json

import datasets


# TODO: Add BibTeX citation
# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@inproceedings{md_gender_bias,
  author    = {Emily Dinan and
               Angela Fan and
               Ledell Wu and
               Jason Weston and
               Douwe Kiela and
               Adina Williams},
  editor    = {Bonnie Webber and
               Trevor Cohn and
               Yulan He and
               Yang Liu},
  title     = {Multi-Dimensional Gender Bias Classification},
  booktitle = {Proceedings of the 2020 Conference on Empirical Methods in Natural
               Language Processing, {EMNLP} 2020, Online, November 16-20, 2020},
  pages     = {314--331},
  publisher = {Association for Computational Linguistics},
  year      = {2020},
  url       = {https://www.aclweb.org/anthology/2020.emnlp-main.23/}
}
"""

# TODO: Add description of the dataset here
# You can copy an official description
_DESCRIPTION = """\
Machine learning models are trained to find patterns in data.
NLP models can inadvertently learn socially undesirable patterns when training on gender biased text.
In this work, we propose a general framework that decomposes gender bias in text along several pragmatic and semantic dimensions:
bias from the gender of the person being spoken about, bias from the gender of the person being spoken to, and bias from the gender of the speaker.
Using this fine-grained framework, we automatically annotate eight large scale datasets with gender information.
In addition, we collect a novel, crowdsourced evaluation benchmark of utterance-level gender rewrites.
Distinguishing between gender bias along multiple dimensions is important, as it enables us to train finer-grained gender bias classifiers.
We show our classifiers prove valuable for a variety of important applications, such as controlling for gender bias in generative models,
detecting gender bias in arbitrary text, and shed light on offensive language in terms of genderedness.
"""

_HOMEPAGE = "https://parl.ai/projects/md_gender/"

_LICENSE = "MIT License"

_URL = "http://parl.ai/downloads/md_gender/gend_multiclass_10072020.tgz"

_CONF_FILES = {
    "funpedia": {
        "train": "funpedia/train.jsonl",
        "validation": "funpedia/valid.jsonl",
        "test": "funpedia/test.jsonl",
    },
    "image_chat": {
        "train": "image_chat/engaging_imagechat_gender_captions_hashed.test.jsonl",
        "validation": "image_chat/engaging_imagechat_gender_captions_hashed.train.jsonl",
        "test": "image_chat/engaging_imagechat_gender_captions_hashed.valid.jsonl",
    },
    "wizard": {
        "train": "wizard/train.jsonl",
        "validation": "wizard/valid.jsonl",
        "test": "wizard/test.jsonl",
    },
    "convai2_inferred": {
        "train": (
            "inferred_about/convai2_train_binary.txt",
            "inferred_about/convai2_train.txt",
        ),
        "validation": (
            "inferred_about/convai2_valid_binary.txt",
            "inferred_about/convai2_valid.txt",
        ),
        "test": (
            "inferred_about/convai2_test_binary.txt",
            "inferred_about/convai2_test.txt",
        ),
    },
    "light_inferred": {
        "train": (
            "inferred_about/light_train_binary.txt",
            "inferred_about/light_train.txt",
        ),
        "validation": (
            "inferred_about/light_valid_binary.txt",
            "inferred_about/light_valid.txt",
        ),
        "test": (
            "inferred_about/light_test_binary.txt",
            "inferred_about/light_test.txt",
        ),
    },
    "opensubtitles_inferred": {
        "train": (
            "inferred_about/opensubtitles_train_binary.txt",
            "inferred_about/opensubtitles_train.txt",
        ),
        "validation": (
            "inferred_about/opensubtitles_valid_binary.txt",
            "inferred_about/opensubtitles_valid.txt",
        ),
        "test": (
            "inferred_about/opensubtitles_test_binary.txt",
            "inferred_about/opensubtitles_test.txt",
        ),
    },
    "yelp_inferred": {
        "train": (
            "inferred_about/yelp_train_binary.txt",
            "",
        ),
        "validation": (
            "inferred_about/yelp_valid_binary.txt",
            "",
        ),
        "test": (
            "inferred_about/yelp_test_binary.txt",
            "",
        ),
    },
}


class MdGenderBias(datasets.GeneratorBasedBuilder):
    """Multi-Dimensional Gender Bias classification"""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="gendered_words",
            version=VERSION,
            description="A list of common nouns with a masculine and feminine variant.",
        ),
        datasets.BuilderConfig(
            name="name_genders",
            version=VERSION,
            description="A list of first names with their gender attribution by year in the US.",
        ),
        datasets.BuilderConfig(
            name="new_data", version=VERSION, description="Some data reformulated and annotated along all three axes."
        ),
        datasets.BuilderConfig(
            name="funpedia",
            version=VERSION,
            description="Data from Funpedia with ABOUT annotations based on Funpedia information on an entity's gender.",
        ),
        datasets.BuilderConfig(
            name="image_chat",
            version=VERSION,
            description="Data from ImageChat with ABOUT annotations based on image recognition.",
        ),
        datasets.BuilderConfig(
            name="wizard",
            version=VERSION,
            description="Data from WizardsOfWikipedia with ABOUT annotations based on Wikipedia information on an entity's gender.",
        ),
        datasets.BuilderConfig(
            name="convai2_inferred",
            version=VERSION,
            description="Data from the ConvAI2 challenge with ABOUT annotations inferred by a trined classifier.",
        ),
        datasets.BuilderConfig(
            name="light_inferred",
            version=VERSION,
            description="Data from LIGHT with ABOUT annotations inferred by a trined classifier.",
        ),
        datasets.BuilderConfig(
            name="opensubtitles_inferred",
            version=VERSION,
            description="Data from OpenSubtitles with ABOUT annotations inferred by a trined classifier.",
        ),
        datasets.BuilderConfig(
            name="yelp_inferred",
            version=VERSION,
            description="Data from Yelp reviews with ABOUT annotations inferred by a trined classifier.",
        ),
    ]

    DEFAULT_CONFIG_NAME = (
        "new_data"  # It's not mandatory to have a default configuration. Just use one if it make sense.
    )

    def _info(self):
        # TODO: This method specifies the datasets.DatasetInfo object which contains informations and typings for the dataset
        if (
            self.config.name == "gendered_words"
        ):  # This is the name of the configuration selected in BUILDER_CONFIGS above
            features = datasets.Features(
                {
                    "word_masculine": datasets.Value("string"),
                    "word_feminine": datasets.Value("string"),
                }
            )
        elif self.config.name == "name_genders":
            features = datasets.Features(
                {
                    "name": datasets.Value("string"),
                    "assigned_gender": datasets.ClassLabel(names=["M", "F"]),
                    "count": datasets.Value("int32"),
                }
            )
        elif self.config.name == "new_data":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "original": datasets.Value("string"),
                    "labels": [
                        datasets.ClassLabel(
                            names=[
                                "ABOUT:female",
                                "ABOUT:male",
                                "PARTNER:female",
                                "PARTNER:male",
                                "SELF:female",
                                "SELF:male",
                            ]
                        )
                    ],
                    "class_type": datasets.ClassLabel(names=["about", "partner", "self"]),
                    "turker_gender": datasets.ClassLabel(
                        names=["man", "woman", "nonbinary", "prefer not to say", "no answer"]
                    ),
                    "episode_done": datasets.Value("bool_"),
                    "confidence": datasets.Value("string"),
                }
            )
        elif self.config.name == "funpedia":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "persona": datasets.Value("string"),
                    "gender": datasets.ClassLabel(names=["gender-neutral", "female", "male"]),
                }
            )
        elif self.config.name == "image_chat":
            features = datasets.Features(
                {
                    "caption": datasets.Value("string"),
                    "id": datasets.Value("string"),
                    "male": datasets.Value("bool_"),
                    "female": datasets.Value("bool_"),
                }
            )
        elif self.config.name == "wizard":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "chosen_topic": datasets.Value("string"),
                    "gender": datasets.ClassLabel(names=["gender-neutral", "female", "male"]),
                }
            )
        elif self.config.name == "yelp_inferred":
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "binary_label": datasets.ClassLabel(names=["ABOUT:female", "ABOUT:male"]),
                    "binary_score": datasets.Value("float"),
                }
            )
        else:  # data with inferred labels
            features = datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "binary_label": datasets.ClassLabel(names=["ABOUT:female", "ABOUT:male"]),
                    "binary_score": datasets.Value("float"),
                    "ternary_label": datasets.ClassLabel(names=["ABOUT:female", "ABOUT:male", "ABOUT:gender-neutral"]),
                    "ternary_score": datasets.Value("float"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,  # Here we define them above because they are different between the two configurations
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        archive = dl_manager.download(_URL)
        data_dir = "data_to_release"
        if self.config.name == "gendered_words":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": None,
                        "filepath_pair": (
                            data_dir + "/" + "word_list/male_word_file.txt",
                            data_dir + "/" + "word_list/female_word_file.txt",
                        ),
                        "files": dl_manager.iter_archive(archive),
                    },
                )
            ]
        elif self.config.name == "name_genders":
            return [
                datasets.SplitGenerator(
                    name=f"yob{yob}",
                    gen_kwargs={
                        "filepath": data_dir + "/" + f"names/yob{yob}.txt",
                        "filepath_pair": None,
                        "files": dl_manager.iter_archive(archive),
                    },
                )
                for yob in range(1880, 2019)
            ]
        elif self.config.name == "new_data":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": data_dir + "/" + "new_data/data.jsonl",
                        "filepath_pair": None,
                        "files": dl_manager.iter_archive(archive),
                    },
                )
            ]
        elif self.config.name in ["funpedia", "image_chat", "wizard"]:
            return [
                datasets.SplitGenerator(
                    name=spl,
                    gen_kwargs={
                        "filepath": data_dir + "/" + fname,
                        "filepath_pair": None,
                        "files": dl_manager.iter_archive(archive),
                    },
                )
                for spl, fname in _CONF_FILES[self.config.name].items()
            ]
        else:
            return [
                datasets.SplitGenerator(
                    name=spl,
                    gen_kwargs={
                        "filepath": None,
                        "filepath_pair": (
                            data_dir + "/" + fname_1,
                            data_dir + "/" + fname_2,
                        ),
                        "files": dl_manager.iter_archive(archive),
                    },
                )
                for spl, (fname_1, fname_2) in _CONF_FILES[self.config.name].items()
            ]

    def _generate_examples(self, filepath, filepath_pair, files):
        if self.config.name == "gendered_words":
            male_data, female_data = None, None
            for path, f in files:
                if path == filepath_pair[0]:
                    male_data = f.read().decode("utf-8").splitlines()
                elif path == filepath_pair[1]:
                    female_data = f.read().decode("utf-8").splitlines()
                if male_data is not None and female_data is not None:
                    for id_, (l_m, l_f) in enumerate(zip(male_data, female_data)):
                        yield id_, {
                            "word_masculine": l_m.strip(),
                            "word_feminine": l_f.strip(),
                        }
                    break
        elif self.config.name == "name_genders":
            for path, f in files:
                if path == filepath:
                    for id_, line in enumerate(f):
                        name, g, ct = line.decode("utf-8").strip().split(",")
                        yield id_, {
                            "name": name,
                            "assigned_gender": g,
                            "count": int(ct),
                        }
                    break
        elif "_inferred" in self.config.name:
            if "yelp" in self.config.name:
                for path, f in files:
                    if path == filepath_pair[0]:
                        for id_, line_b in enumerate(f):
                            text_b, label_b, score_b = line_b.decode("utf-8").split("\t")
                            yield id_, {
                                "text": text_b,
                                "binary_label": label_b,
                                "binary_score": float(score_b.strip()),
                            }
                        break
            else:
                binary_data, ternary_data = None, None
                for path, f in files:
                    if path == filepath_pair[0]:
                        binary_data = f.read().decode("utf-8").splitlines()
                    elif path == filepath_pair[1]:
                        ternary_data = f.read().decode("utf-8").splitlines()
                    if binary_data is not None and ternary_data is not None:
                        for id_, (line_b, line_t) in enumerate(zip(binary_data, ternary_data)):
                            text_b, label_b, score_b = line_b.split("\t")
                            text_t, label_t, score_t = line_t.split("\t")
                            yield id_, {
                                "text": text_b,
                                "binary_label": label_b,
                                "binary_score": float(score_b.strip()),
                                "ternary_label": label_t,
                                "ternary_score": float(score_t.strip()),
                            }
                        break
        else:
            for path, f in files:
                if path == filepath:
                    for id_, line in enumerate(f):
                        example = json.loads(line.decode("utf-8").strip())
                        if "turker_gender" in example and example["turker_gender"] is None:
                            example["turker_gender"] = "no answer"
                        yield id_, example
                    break



"""An Evaluation Dataset for Intent Classification and Out-of-Scope Prediction"""


import json
import textwrap

import datasets


_CITATION = """\
    @inproceedings{larson-etal-2019-evaluation,
    title = "An Evaluation Dataset for Intent Classification and Out-of-Scope Prediction",
    author = "Larson, Stefan  and
      Mahendran, Anish  and
      Peper, Joseph J.  and
      Clarke, Christopher  and
      Lee, Andrew  and
      Hill, Parker  and
      Kummerfeld, Jonathan K.  and
      Leach, Kevin  and
      Laurenzano, Michael A.  and
      Tang, Lingjia  and
      Mars, Jason",
    booktitle = "Proceedings of the 2019 Conference on Empirical Methods in Natural Language Processing and the 9th International Joint Conference on Natural Language Processing (EMNLP-IJCNLP)",
    year = "2019",
    url = "https://www.aclweb.org/anthology/D19-1131"
}
"""

_DESCRIPTION = """\
    This dataset is for evaluating the performance of intent classification systems in the
    presence of "out-of-scope" queries. By "out-of-scope", we mean queries that do not fall
    into any of the system-supported intent classes. Most datasets include only data that is
    "in-scope". Our dataset includes both in-scope and out-of-scope data. You might also know
    the term "out-of-scope" by other terms, including "out-of-domain" or "out-of-distribution".
"""

_DESCRIPTIONS = {
    "small": textwrap.dedent(
        """\
        Small, in which there are only 50 training queries per each in-scope intent
        """
    ),
    "imbalanced": textwrap.dedent(
        """\
        Imbalanced, in which intents have either 25, 50, 75, or 100 training queries.
        """
    ),
    "plus": textwrap.dedent(
        """\
        OOS+, in which there are 250 out-of-scope training examples, rather than 100.
        """
    ),
}

_URL = "https://github.com/clinc/oos-eval/"

_DATA_URLS = {
    "small": "https://raw.githubusercontent.com/clinc/oos-eval/master/data/data_small.json",
    "imbalanced": "https://raw.githubusercontent.com/clinc/oos-eval/master/data/data_imbalanced.json",
    "plus": "https://raw.githubusercontent.com/clinc/oos-eval/master/data/data_oos_plus.json",
}


class ClincConfig(datasets.BuilderConfig):

    """BuilderConfig for CLINC150"""

    def __init__(self, description, data_url, citation, url, **kwrags):
        """
        Args:
            description: `string`, brief description of the dataset
            data_url: `dictionary`, dict with url for each split of data.
            citation: `string`, citation for the dataset.
            url: `string`, url for information about the dataset.
            **kwrags: keyword arguments frowarded to super
        """
        super(ClincConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwrags)
        self.description = description
        self.data_url = data_url
        self.citation = citation
        self.url = url


class ClincOos(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        ClincConfig(
            name=name, description=_DESCRIPTIONS[name], data_url=_DATA_URLS[name], citation=_CITATION, url=_URL
        )
        for name in ["small", "imbalanced", "plus"]
    ]

    def _info(self):
        features = {}
        features["text"] = datasets.Value("string")
        labels_list = [
            "restaurant_reviews",
            "nutrition_info",
            "account_blocked",
            "oil_change_how",
            "time",
            "weather",
            "redeem_rewards",
            "interest_rate",
            "gas_type",
            "accept_reservations",
            "smart_home",
            "user_name",
            "report_lost_card",
            "repeat",
            "whisper_mode",
            "what_are_your_hobbies",
            "order",
            "jump_start",
            "schedule_meeting",
            "meeting_schedule",
            "freeze_account",
            "what_song",
            "meaning_of_life",
            "restaurant_reservation",
            "traffic",
            "make_call",
            "text",
            "bill_balance",
            "improve_credit_score",
            "change_language",
            "no",
            "measurement_conversion",
            "timer",
            "flip_coin",
            "do_you_have_pets",
            "balance",
            "tell_joke",
            "last_maintenance",
            "exchange_rate",
            "uber",
            "car_rental",
            "credit_limit",
            "oos",
            "shopping_list",
            "expiration_date",
            "routing",
            "meal_suggestion",
            "tire_change",
            "todo_list",
            "card_declined",
            "rewards_balance",
            "change_accent",
            "vaccines",
            "reminder_update",
            "food_last",
            "change_ai_name",
            "bill_due",
            "who_do_you_work_for",
            "share_location",
            "international_visa",
            "calendar",
            "translate",
            "carry_on",
            "book_flight",
            "insurance_change",
            "todo_list_update",
            "timezone",
            "cancel_reservation",
            "transactions",
            "credit_score",
            "report_fraud",
            "spending_history",
            "directions",
            "spelling",
            "insurance",
            "what_is_your_name",
            "reminder",
            "where_are_you_from",
            "distance",
            "payday",
            "flight_status",
            "find_phone",
            "greeting",
            "alarm",
            "order_status",
            "confirm_reservation",
            "cook_time",
            "damaged_card",
            "reset_settings",
            "pin_change",
            "replacement_card_duration",
            "new_card",
            "roll_dice",
            "income",
            "taxes",
            "date",
            "who_made_you",
            "pto_request",
            "tire_pressure",
            "how_old_are_you",
            "rollover_401k",
            "pto_request_status",
            "how_busy",
            "application_status",
            "recipe",
            "calendar_update",
            "play_music",
            "yes",
            "direct_deposit",
            "credit_limit_change",
            "gas",
            "pay_bill",
            "ingredients_list",
            "lost_luggage",
            "goodbye",
            "what_can_i_ask_you",
            "book_hotel",
            "are_you_a_bot",
            "next_song",
            "change_speed",
            "plug_type",
            "maybe",
            "w2",
            "oil_change_when",
            "thank_you",
            "shopping_list_update",
            "pto_balance",
            "order_checks",
            "travel_alert",
            "fun_fact",
            "sync_device",
            "schedule_maintenance",
            "apr",
            "transfer",
            "ingredient_substitution",
            "calories",
            "current_location",
            "international_fees",
            "calculator",
            "definition",
            "next_holiday",
            "update_playlist",
            "mpg",
            "min_payment",
            "change_user_name",
            "restaurant_suggestion",
            "travel_notification",
            "cancel",
            "pto_used",
            "travel_suggestion",
            "change_volume",
        ]
        features["intent"] = datasets.ClassLabel(names=labels_list)

        return datasets.DatasetInfo(
            description=_DESCRIPTION + "\n" + self.config.description,
            features=datasets.Features(features),
            homepage=self.config.url,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        file_ = dl_manager.download_and_extract(self.config.data_url)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": file_, "split": "train"}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": file_, "split": "val"}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": file_, "split": "test"}),
        ]

    def _generate_examples(self, filepath, split):
        with open(filepath, encoding="utf-8") as f:
            j = json.load(f)
            for id_, row in enumerate(j[split] + j["oos_" + split]):
                yield id_, {"text": row[0], "intent": row[1]}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""\
This dataset gathers 728,321 biographies from Wikipedia. It aims at evaluating text generation
algorithms. For each article, we provide the first paragraph and the infobox.
"""


import os

import datasets


_CITATION = """\
@article{DBLP:journals/corr/LebretGA16,
  author    = {R{\'{e}}mi Lebret and
               David Grangier and
               Michael Auli},
  title     = {Generating Text from Structured Data with Application to the Biography
               Domain},
  journal   = {CoRR},
  volume    = {abs/1603.07771},
  year      = {2016},
  url       = {http://arxiv.org/abs/1603.07771},
  archivePrefix = {arXiv},
  eprint    = {1603.07771},
  timestamp = {Mon, 13 Aug 2018 16:48:30 +0200},
  biburl    = {https://dblp.org/rec/journals/corr/LebretGA16.bib},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_DESCRIPTION = """\
This dataset gathers 728,321 biographies from wikipedia. It aims at evaluating text generation
algorithms. For each article, we provide the first paragraph and the infobox (both tokenized).
For each article, we extracted the first paragraph (text), the infobox (structured data). Each
infobox is encoded as a list of (field name, field value) pairs. We used Stanford CoreNLP
(http://stanfordnlp.github.io/CoreNLP/) to preprocess the data, i.e. we broke the text into
sentences and tokenized both the text and the field values. The dataset was randomly split in
three subsets train (80%), valid (10%), test (10%).
"""

_HOMEPAGE = "https://github.com/DavidGrangier/wikipedia-biography-dataset"

_LICENSE = "CC BY-SA 3.0"

# The HuggingFace dataset library don't host the datasets but only point to the original files
# This can be an arbitrary nested dict/list of URLs (see below in `_split_generators` method)
_URL = "https://drive.google.com/uc?export=download&id=1L7aoUXzHPzyzQ0ns4ApBbYepsjFOtXil"


def _get_table(infobox_line):
    """Converts the infobox into a one row table."""
    cells = infobox_line.split("\t")
    # remove empty cells
    cells = list(filter(lambda x: x.find("<none>") == -1, cells))
    columns = set([cell[0 : cell.split(":")[0].rfind("_")] for cell in cells])
    table = {col: dict() for col in columns}
    for cell in cells:
        delimiter_position_value = cell.find(":")
        column_index = cell[0:delimiter_position_value]
        value = cell[delimiter_position_value + 1 :]
        delimiter_column_index = column_index.rfind("_")
        column = column_index[0:delimiter_column_index]
        index = column_index[delimiter_column_index + 1 :]
        table[column][index] = value
    infobox_line_as_table = []
    for column in table.keys():
        row_value = " ".join([table[column][index] for index in sorted(table[column].keys())])
        infobox_line_as_table.append(
            {
                "column_header": column,
                "row_number": 1,
                "content": row_value,
            }
        )
    return infobox_line_as_table


class WikiBio(datasets.GeneratorBasedBuilder):
    """Infoboxes and first paragraph from Wikipedia biography pages."""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        features = datasets.Features(
            {
                "input_text": {
                    "table": datasets.Sequence(
                        {
                            "column_header": datasets.Value("string"),
                            "row_number": datasets.Value("int16"),
                            "content": datasets.Value("string"),
                        }
                    ),
                    "context": datasets.Value("string"),
                },
                "target_text": datasets.Value("string"),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=("input_text", "target_text"),
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URL
        data_dir = dl_manager.download_and_extract(my_urls)
        data_path = os.path.join(data_dir, "wikipedia-biography-dataset")
        return [
            datasets.SplitGenerator(
                name=datasets.Split("train"),
                gen_kwargs={
                    "id_file": os.path.join(data_path, "train", "train.id"),
                    "infobox_file": os.path.join(data_path, "train", "train.box"),
                    "nb_lines_file": os.path.join(data_path, "train", "train.nb"),
                    "sentences_file": os.path.join(data_path, "train", "train.sent"),
                    "article_title_file": os.path.join(data_path, "train", "train.title"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("test"),
                gen_kwargs={
                    "id_file": os.path.join(data_path, "test", "test.id"),
                    "infobox_file": os.path.join(data_path, "test", "test.box"),
                    "nb_lines_file": os.path.join(data_path, "test", "test.nb"),
                    "sentences_file": os.path.join(data_path, "test", "test.sent"),
                    "article_title_file": os.path.join(data_path, "test", "test.title"),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("val"),
                gen_kwargs={
                    "id_file": os.path.join(data_path, "valid", "valid.id"),
                    "infobox_file": os.path.join(data_path, "valid", "valid.box"),
                    "nb_lines_file": os.path.join(data_path, "valid", "valid.nb"),
                    "sentences_file": os.path.join(data_path, "valid", "valid.sent"),
                    "article_title_file": os.path.join(data_path, "valid", "valid.title"),
                },
            ),
        ]

    def _generate_examples(self, id_file, infobox_file, nb_lines_file, sentences_file, article_title_file):
        """Yields examples."""
        with open(id_file, "r", encoding="utf-8") as id_src, open(
            infobox_file, "r", encoding="utf-8"
        ) as infobox_src, open(nb_lines_file, "r", encoding="utf-8") as nb_lines_src, open(
            sentences_file, "r", encoding="utf-8"
        ) as sentences_src, open(
            article_title_file, "r", encoding="utf-8"
        ) as article_title_src:
            for id_, infobox, nb_lines, article_title in zip(id_src, infobox_src, nb_lines_src, article_title_src):
                target_text = []
                for _ in range(int(nb_lines)):
                    target_text.append(sentences_src.readline())
                yield id_, {
                    "input_text": {"table": _get_table(infobox), "context": article_title},
                    "target_text": "".join(target_text),
                }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""OneStopEnglish Corpus: Dataset of texts classified into reading levels/text complexities."""


import os

import datasets
from datasets.tasks import TextClassification


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{vajjala-lucic-2018-onestopenglish,
    title = {OneStopEnglish corpus: A new corpus for automatic readability assessment and text simplification},
    author = {Sowmya Vajjala and Ivana Lučić},
    booktitle = {Proceedings of the Thirteenth Workshop on Innovative Use of NLP for Building Educational Applications},
    year = {2018}
}
"""

_DESCRIPTION = """\
This dataset is a compilation of the OneStopEnglish corpus of texts written at three reading levels into one file.
Text documents are classified into three reading levels - ele, int, adv (Elementary, Intermediate and Advance).
This dataset demonstrates its usefulness for through two applica-tions - automatic  readability  assessment  and automatic text simplification.
The corpus consists of 189 texts, each in three versions/reading levels (567 in total).
"""

_HOMEPAGE = "https://github.com/nishkalavallabhi/OneStopEnglishCorpus"

_LICENSE = "Creative Commons Attribution-ShareAlike 4.0 International License"

_URL = "https://github.com/purvimisal/OneStopCorpus-Compiled/raw/main/Texts-SeparatedByReadingLevel.zip"


# TODO: Name of the dataset usually match the script name with CamelCase instead of snake_case
class OnestopEnglish(datasets.GeneratorBasedBuilder):
    """OneStopEnglish Corpus: Dataset of texts classified into reading levels"""

    VERSION = datasets.Version("1.1.0")

    def _info(self):
        # TODO: This method specifies the datasets.DatasetInfo object which contains informations and typings for the dataset
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {"text": datasets.Value("string"), "label": datasets.features.ClassLabel(names=["ele", "int", "adv"])}
            ),
            supervised_keys=[""],
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="label")],
        )

    def _vocab_text_gen(self, train_file):
        for _, ex in self._generate_examples(train_file):
            yield ex["text"]

    def _split_generators(self, dl_manager):
        """Downloads OneStopEnglish corpus"""
        extracted_folder_path = dl_manager.download_and_extract(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"split_key": "train", "data_dir": extracted_folder_path},
            )
        ]

    def _get_examples_from_split(self, split_key, data_dir):
        """Reads the downloaded and extracted files and combines the individual text files to one dataset."""

        data_dir = os.path.join(data_dir, "Texts-SeparatedByReadingLevel")

        ele_samples = []
        dir_path = os.path.join(data_dir, "Ele-Txt")
        files = os.listdir(dir_path)
        for f in sorted(files):
            try:
                with open(os.path.join(dir_path, f), encoding="utf-8-sig") as myfile:
                    text = myfile.read().strip()
                    ele_samples.append(text)
            except Exception as e:
                logger.info("Error with:", os.path.join(dir_path, f), e)

        int_samples = []
        dir_path = os.path.join(data_dir, "Int-Txt")
        files = os.listdir(dir_path)
        for f in sorted(files):
            try:
                with open(os.path.join(dir_path, f), encoding="utf-8-sig") as myfile:
                    text = myfile.read().strip()
                    int_samples.append(text)
            except Exception as e:
                logger.info("Error with:", os.path.join(dir_path, f), e)

        adv_samples = []
        dir_path = os.path.join(data_dir, "Adv-Txt")
        files = os.listdir(dir_path)
        for f in sorted(files):
            try:
                with open(os.path.join(dir_path, f), encoding="utf-8-sig") as myfile:
                    text = myfile.read().strip()
                    adv_samples.append(text)
            except Exception as e:
                logger.info("Error with:", os.path.join(dir_path, f), e)

        train_samples = ele_samples + int_samples + adv_samples
        train_labels = (["ele"] * len(ele_samples)) + (["int"] * len(int_samples)) + (["adv"] * len(adv_samples))

        if split_key == "train":
            return (train_samples, train_labels)
        else:
            raise ValueError(f"Invalid split key {split_key}")

    def _generate_examples(self, split_key, data_dir):
        """Yields examples for a given split of dataset."""
        split_text, split_labels = self._get_examples_from_split(split_key, data_dir)
        for id_, (text, label) in enumerate(zip(split_text, split_labels)):
            feature_dict = {"text": text, "label": label}
            yield id_, feature_dict



"""TODO(hellaswag): Add a description here."""


import json

import datasets


# TODO(hellaswag): BibTeX citation
_CITATION = """\
@inproceedings{zellers2019hellaswag,
    title={HellaSwag: Can a Machine Really Finish Your Sentence?},
    author={Zellers, Rowan and Holtzman, Ari and Bisk, Yonatan and Farhadi, Ali and Choi, Yejin},
    booktitle ={Proceedings of the 57th Annual Meeting of the Association for Computational Linguistics},
    year={2019}
}
"""

# TODO(hellaswag):
_DESCRIPTION = """
"""
_URL = "https://github.com/rowanz/hellaswag/raw/master/data/"
_URLS = {
    "train": _URL + "hellaswag_train.jsonl",
    "test": _URL + "hellaswag_test.jsonl",
    "dev": _URL + "hellaswag_val.jsonl",
}


class Hellaswag(datasets.GeneratorBasedBuilder):
    """TODO(hellaswag): Short description of my dataset."""

    # TODO(hellaswag): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(hellaswag): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    "ind": datasets.Value("int32"),
                    "activity_label": datasets.Value("string"),
                    "ctx_a": datasets.Value("string"),
                    "ctx_b": datasets.Value("string"),
                    "ctx": datasets.Value("string"),
                    "endings": datasets.features.Sequence(datasets.Value("string")),
                    "source_id": datasets.Value("string"),
                    "split": datasets.Value("string"),
                    "split_type": datasets.Value("string"),
                    "label": datasets.Value("string"),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://rowanzellers.com/hellaswag/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(hellaswag): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        urls_to_download = _URLS
        dl_dir = dl_manager.download_and_extract(urls_to_download)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": dl_dir["train"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": dl_dir["test"]},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": dl_dir["dev"]},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(hellaswag): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                yield id_, {
                    "ind": int(data["ind"]),
                    "activity_label": data["activity_label"],
                    "ctx_a": data.get("ctx_a", ""),
                    "ctx_b": data.get("ctx_b", ""),
                    "ctx": data["ctx"],
                    "endings": data.get("endings", []),
                    "source_id": data["source_id"],
                    "split": data["split"],
                    "split_type": data["split_type"],
                    "label": str(data.get("label", "")),
                }



"""TODO(fquad): Add a description here."""


import json
import os

import datasets


# TODO(fquad): BibTeX citation
_CITATION = """\
@ARTICLE{2020arXiv200206071
       author = {Martin, d'Hoffschmidt and Maxime, Vidal and
         Wacim, Belblidia and Tom, Brendlé},
        title = "{FQuAD: French Question Answering Dataset}",
      journal = {arXiv e-prints},
     keywords = {Computer Science - Computation and Language},
         year = "2020",
        month = "Feb",
          eid = {arXiv:2002.06071},
        pages = {arXiv:2002.06071},
archivePrefix = {arXiv},
       eprint = {2002.06071},
 primaryClass = {cs.CL}
}
"""

# TODO(fquad):
_DESCRIPTION = """\
FQuAD: French Question Answering Dataset
We introduce FQuAD, a native French Question Answering Dataset. FQuAD contains 25,000+ question and answer pairs.
Finetuning CamemBERT on FQuAD yields a F1 score of 88% and an exact match of 77.9%.

"""

_URL = "https://storage.googleapis.com/illuin/fquad/"
_URLS = {
    "train": _URL + "train.json.zip",
    "valid": _URL + "valid.json.zip",
}


class Fquad(datasets.GeneratorBasedBuilder):
    """TODO(fquad): Short description of my dataset."""

    # TODO(fquad): Set up version.
    VERSION = datasets.Version("0.1.0")

    def _info(self):
        # TODO(fquad): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "context": datasets.Value("string"),
                    "questions": datasets.features.Sequence(datasets.Value("string")),
                    "answers": datasets.features.Sequence(
                        {"texts": datasets.Value("string"), "answers_starts": datasets.Value("int32")}
                    ),
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://fquad.illuin.tech/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(fquad): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        download_urls = _URLS
        dl_dir = dl_manager.download_and_extract(download_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_dir["train"], "train.json")},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(dl_dir["valid"], "valid.json")},
            ),
        ]

    def _generate_examples(self, filepath):

        """Yields examples."""
        # TODO(fquad): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for id1, examples in enumerate(data["data"]):
                for id2, example in enumerate(examples["paragraphs"]):
                    questions = [question["question"] for question in example["qas"]]
                    answers = [answer["answers"] for answer in example["qas"]]
                    texts = [answer[0]["text"] for answer in answers]
                    answers_starts = [answer[0]["answer_start"] for answer in answers]

                    yield str(id1) + "_" + str(id2), {
                        "context": example["context"],
                        "questions": questions,
                        "answers": {"texts": texts, "answers_starts": answers_starts},
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This dataset is a new knowledge-base (KB) of hasPart relationships, extracted from a large corpus of generic statements. Complementary to other resources available, it is the first which is all three of: accurate (90% precision), salient (covers relationships a person may mention), and has high coverage of common terms (approximated as within a 10 year old’s vocabulary), as well as having several times more hasPart entries than in the popular ontologies ConceptNet and WordNet. In addition, it contains information about quantifiers, argument modifiers, and links the entities to appropriate concepts in Wikipedia and WordNet."""


import ast
from collections import defaultdict

import datasets


_CITATION = """\
@misc{bhakthavatsalam2020dogs,
      title={Do Dogs have Whiskers? A New Knowledge Base of hasPart Relations},
      author={Sumithra Bhakthavatsalam and Kyle Richardson and Niket Tandon and Peter Clark},
      year={2020},
      eprint={2006.07510},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
This dataset is a new knowledge-base (KB) of hasPart relationships, extracted from a large corpus of generic statements. Complementary to other resources available, it is the first which is all three of: accurate (90% precision), salient (covers relationships a person may mention), and has high coverage of common terms (approximated as within a 10 year old’s vocabulary), as well as having several times more hasPart entries than in the popular ontologies ConceptNet and WordNet. In addition, it contains information about quantifiers, argument modifiers, and links the entities to appropriate concepts in Wikipedia and WordNet.
"""

_HOMEPAGE = "https://allenai.org/data/haspartkb"

_LICENSE = ""


TSV_ID = "1Ev4RqWcPsLI9rgOGAKh-_dFKqcEZ1u-G"
FOLDER_ID = "1NzjXX46NnpxtgxBrkBWFiUbsXAMdd-lB"
ID = TSV_ID

_URL = f"https://drive.google.com/uc?export=download&id={ID}"


class HasPart(datasets.GeneratorBasedBuilder):
    def _info(self):
        features = datasets.Features(
            {
                "arg1": datasets.features.Value("string"),
                "arg2": datasets.features.Value("string"),
                "score": datasets.features.Value("float64"),
                "wikipedia_primary_page": datasets.features.Sequence(datasets.features.Value("string")),
                "synset": datasets.features.Sequence(datasets.features.Value("string")),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        dl_fp = dl_manager.download_and_extract(_URL)

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "input_file": dl_fp,
                    "split": "train",
                },
            ),
        ]

    def _parse_metadata(self, md):
        """metadata is a list of dicts in the tsv file, hence needs to be parsed using
        ast.literal_eval to convert to python objects.

        Note that metadata resulting in parsing error will be skipped
        """
        md = ast.literal_eval(md)
        dd = defaultdict(list)

        for entry in md:
            try:
                for k, v in entry.items():
                    dd[k].append(v)
            except AttributeError:
                continue
        return dd

    def _generate_examples(self, input_file, split):
        """Yields examples."""
        with open(input_file, encoding="utf-8") as f:
            for id_, line in enumerate(f):
                _, arg1, arg2, score, metadata = line.split("\t")
                metadata = self._parse_metadata(metadata)
                example = {
                    "arg1": arg1,
                    "arg2": arg2,
                    "score": float(score),
                    "wikipedia_primary_page": metadata["wikipedia_primary_page"],
                    "synset": metadata["synset"],
                }
                yield id_, example



"""TODO(winogrande): Add a description here."""


import json
import os

import datasets


# TODO(winogrande): BibTeX citation
_CITATION = """\
@InProceedings{ai2:winogrande,
title = {WinoGrande: An Adversarial Winograd Schema Challenge at Scale},
authors={Keisuke, Sakaguchi and Ronan, Le Bras and Chandra, Bhagavatula and Yejin, Choi
},
year={2019}
}
"""

# TODO(winogrande):
_DESCRIPTION = """\
WinoGrande is a new collection of 44k problems, inspired by Winograd Schema Challenge (Levesque, Davis, and Morgenstern
 2011), but adjusted to improve the scale and robustness against the dataset-specific bias. Formulated as a
fill-in-a-blank task with binary options, the goal is to choose the right option for a given sentence which requires
commonsense reasoning.
"""

_URL = "https://storage.googleapis.com/ai2-mosaic/public/winogrande/winogrande_1.1.zip"
_FORMATS = ["xs", "s", "m", "l", "xl", "debiased"]


class WinograndeConfig(datasets.BuilderConfig):

    """BuilderConfig for Discofuse"""

    def __init__(self, data_size, **kwargs):
        """

        Args:
            data_size: the format of the training set we want to use (xs, s, m, l, xl, debiased)
            **kwargs: keyword arguments forwarded to super.
        """
        super(WinograndeConfig, self).__init__(version=datasets.Version("1.1.0", ""), **kwargs)
        self.data_size = data_size


class Winogrande(datasets.GeneratorBasedBuilder):
    """TODO(winogrande): Short description of my dataset."""

    # TODO(winogrande): Set up version.
    VERSION = datasets.Version("1.1.0")
    BUILDER_CONFIGS = [
        WinograndeConfig(name="winogrande_" + data_size, description="AI2 dataset", data_size=data_size)
        for data_size in _FORMATS
    ]

    def _info(self):
        # TODO(winogrande): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "option1": datasets.Value("string"),
                    "option2": datasets.Value("string"),
                    "answer": datasets.Value("string")
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://leaderboard.allenai.org/winogrande/submissions/get-started",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(winogrande): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(dl_dir, "winogrande_1.1")
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, f"train_{self.config.data_size}.jsonl"),
                    # 'labelpath': os.path.join(data_dir, 'train_{}-labels.lst'.format(self.config.data_size)),
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(data_dir, "test.jsonl"), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": os.path.join(data_dir, "dev.jsonl"),
                    # 'labelpath': os.path.join(data_dir, 'dev-labels.lst'),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        # TODO(winogrande): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            for id_, row in enumerate(f):
                data = json.loads(row)
                if split == "test":
                    yield id_, {
                        "sentence": data["sentence"],
                        "option1": data["option1"],
                        "option2": data["option2"],
                        "answer": "",
                    }
                else:
                    yield id_, {
                        "sentence": data["sentence"],
                        "option1": data["option1"],
                        "option2": data["option2"],
                        "answer": data["answer"],
                    }


# def _generate_test_example(filepath, split, labelpath=None):
#       with open(filepath, encoding="utf-8") as f:
#           for id_, row in enumerate(f):
#               data = json.loads(row)
#               yield id_,{
#                   'sentence': data['sentence'],
#                   'option1': data['option1'],
#                   'option2': data['option2'],
#                   'answer': None
#               }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import os

import datasets


_DESCRIPTION = """\
Parallel corpora from Web Crawls collected in the ParaCrawl project and further processed for making it a multi-parallel corpus by pivoting via English. Here we only provide the additional language pairs that came out of pivoting. The bitexts for English are available from the ParaCrawl release.
40 languages, 669 bitexts
total number of files: 40
total number of tokens: 10.14G
total number of sentence fragments: 505.48M

Please, acknowledge the ParaCrawl project at http://paracrawl.eu. This version is derived from the original release at their website adjusted for redistribution via the OPUS corpus collection. Please, acknowledge OPUS as well for this service.
"""
_HOMEPAGE_URL = "http://opus.nlpl.eu/MultiParaCrawl.php"
_CITATION = """\
@InProceedings{TIEDEMANN12.463,
  author = {J�rg Tiedemann},
  title = {Parallel Data, Tools and Interfaces in OPUS},
  booktitle = {Proceedings of the Eight International Conference on Language Resources and Evaluation (LREC'12)},
  year = {2012},
  month = {may},
  date = {23-25},
  address = {Istanbul, Turkey},
  editor = {Nicoletta Calzolari (Conference Chair) and Khalid Choukri and Thierry Declerck and Mehmet Ugur Dogan and Bente Maegaard and Joseph Mariani and Jan Odijk and Stelios Piperidis},
  publisher = {European Language Resources Association (ELRA)},
  isbn = {978-2-9517408-7-7},
  language = {english}
 }
"""

_VERSION = "7.1.0"
_BASE_NAME = "MultiParaCrawl.{}.{}"
_BASE_URL = "https://object.pouta.csc.fi/OPUS-MultiParaCrawl/v7.1/moses/{}-{}.txt.zip"

# Please note that only few pairs are shown here. You can use config to generate data for all language pairs
_LANGUAGE_PAIRS = [
    ("cs", "is"),
    ("ga", "sk"),
    ("lv", "mt"),
    ("nb", "ru"),
    ("de", "tl"),
]


class MultiParaCrawlConfig(datasets.BuilderConfig):
    def __init__(self, *args, lang1=None, lang2=None, **kwargs):
        super().__init__(
            *args,
            name=f"{lang1}-{lang2}",
            **kwargs,
        )
        self.lang1 = lang1
        self.lang2 = lang2


class MultiParaCrawl(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        MultiParaCrawlConfig(
            lang1=lang1,
            lang2=lang2,
            description=f"Translating {lang1} to {lang2} or vice versa",
            version=datasets.Version(_VERSION),
        )
        for lang1, lang2 in _LANGUAGE_PAIRS
    ]
    BUILDER_CONFIG_CLASS = MultiParaCrawlConfig

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "translation": datasets.Translation(languages=(self.config.lang1, self.config.lang2)),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        def _base_url(lang1, lang2):
            return _BASE_URL.format(lang1, lang2)

        download_url = _base_url(self.config.lang1, self.config.lang2)
        path = dl_manager.download_and_extract(download_url)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": path},
            )
        ]

    def _generate_examples(self, datapath):
        l1, l2 = self.config.lang1, self.config.lang2
        folder = l1 + "-" + l2
        l1_file = _BASE_NAME.format(folder, l1)
        l2_file = _BASE_NAME.format(folder, l2)
        l1_path = os.path.join(datapath, l1_file)
        l2_path = os.path.join(datapath, l2_file)
        with open(l1_path, encoding="utf-8") as f1, open(l2_path, encoding="utf-8") as f2:
            for sentence_counter, (x, y) in enumerate(zip(f1, f2)):
                x = x.strip()
                y = y.strip()
                result = (
                    sentence_counter,
                    {
                        "id": str(sentence_counter),
                        "translation": {l1: x, l2: y},
                    },
                )
                yield result
